︠567b4d3b-16d1-4a11-b93f-eaca8e15b951i︠
%html
<p style="text-align: justify;">Note that an <a title="Update: How to implement basic algebraic structures" href="http://flask.sagenb.org/home/pub/82/">updated version</a> of this worksheet is available in the new "flask notebook server".</p>
<h1 style="text-align: center;">How to implement basic algebraic structures</h1>
<address><span style="font-size: small;">Simon King</span></address><address><span style="font-size: small;">Friedrich-Schiller-Universit&auml;t Jena</span></address><address><span style="font-size: small;">E-mail: simon dot king at uni hyphen jena dot de</span></address><address><span style="font-size: small;">&copy;&nbsp;2011</span></address><address><br /></address>
<p>The aim of this tutorial is to explain how one can benefit from Sage's category framework and coercion model when implementing new algebraic structures.</p>
<p>On the one hand, we try to explain the theoretical background. On the other hand, we beef up the theory with a sample implementation of fraction fields. The example is quite thorough and detailed. It covers Sage's category framework, the construction of unique parents, and both simple and advanced parts of Sage's coercion model.</p>
<p>The example is developed step by step, so that it should be easy to follow. Not all applications will require all details that we expose here, so that the reader may pick the cherries that are needed for her or his own work.</p>
<h2>Chosing a base class</h2>
<p>In Sage, objects that "contain" other objects and belong to a mathematical category (such as the category of sets or the category of algebras over the rational field) are called Parent. Hence, a parent in Sage and an object of a category are very similar notions. A member of a parent is called an element.</p>
<p>When implementing a parent in Sage, one should start with a class that inherits from :class:`sage.structure.parent.Parent`. Note that there exist some very old parents in Sage that are derived from other classes. We aim at refactoring them, but there needs more work to be done (volunteers welcome). Similarly, when implementing an element in Sage, one should start with a class that inherits from :class:' sage.structure.element.Element`.</p>
<p>Sage also provides certain sub-classes that provide the basic framework for a variety of more concrete algebraic parents, such as groups, rings, or fields, and of their elements.</p>
<h3>The parent</h3>
<p>We want to implement a field, and we should thus start with the base class :class:`sage.rings.field.Field`.</p>

︡1f4f247b-13d3-4596-9cae-4062135cac4e︡{"html": "<p style=\"text-align: justify;\">Note that an <a title=\"Update: How to implement basic algebraic structures\" href=\"http://flask.sagenb.org/home/pub/82/\">updated version</a> of this worksheet is available in the new \"flask notebook server\".</p>\n<h1 style=\"text-align: center;\">How to implement basic algebraic structures</h1>\n<address><span style=\"font-size: small;\">Simon King</span></address><address><span style=\"font-size: small;\">Friedrich-Schiller-Universit&auml;t Jena</span></address><address><span style=\"font-size: small;\">E-mail: simon dot king at uni hyphen jena dot de</span></address><address><span style=\"font-size: small;\">&copy;&nbsp;2011</span></address><address><br /></address>\n<p>The aim of this tutorial is to explain how one can benefit from Sage's category framework and coercion model when implementing new algebraic structures.</p>\n<p>On the one hand, we try to explain the theoretical background. On the other hand, we beef up the theory with a sample implementation of fraction fields. The example is quite thorough and detailed. It covers Sage's category framework, the construction of unique parents, and both simple and advanced parts of Sage's coercion model.</p>\n<p>The example is developed step by step, so that it should be easy to follow. Not all applications will require all details that we expose here, so that the reader may pick the cherries that are needed for her or his own work.</p>\n<h2>Chosing a base class</h2>\n<p>In Sage, objects that \"contain\" other objects and belong to a mathematical category (such as the category of sets or the category of algebras over the rational field) are called Parent. Hence, a parent in Sage and an object of a category are very similar notions. A member of a parent is called an element.</p>\n<p>When implementing a parent in Sage, one should start with a class that inherits from :class:`sage.structure.parent.Parent`. Note that there exist some very old parents in Sage that are derived from other classes. We aim at refactoring them, but there needs more work to be done (volunteers welcome). Similarly, when implementing an element in Sage, one should start with a class that inherits from :class:' sage.structure.element.Element`.</p>\n<p>Sage also provides certain sub-classes that provide the basic framework for a variety of more concrete algebraic parents, such as groups, rings, or fields, and of their elements.</p>\n<h3>The parent</h3>\n<p>We want to implement a field, and we should thus start with the base class :class:`sage.rings.field.Field`.</p>"}︡
︠555a3ca0-b4ae-4306-981f-a03ad6b2ef65︠
from sage.rings.field import Field
︡fce61578-2901-4e70-ac4b-2cb5ed36e3a1︡︡
︠3f7ca468-0cd0-4059-abc3-8f9598489611i︠
%html
<p>Let us compare the methods provided by that class with the methods provided by a general parent:</p>

︡07ddc6d6-6c36-4cc6-be8e-01b96e339cd4︡{"html": "<p>Let us compare the methods provided by that class with the methods provided by a general parent:</p>"}︡
︠32410346-9fc2-482a-af7b-2241b6cb4cc0︠
[p for p in dir(Field) if p not in dir(Parent)]
︡83e3429f-8d14-4046-ab48-560e61c418f3︡{"stdout": "['__div__', '__fraction_field', '__ideal_monoid', '__iter__', '__mul__', '__pow__', '__rdiv__', '__rmul__', '__rpow__', '__rxor__', '__xor__', '_an_element', '_an_element_c', '_an_element_impl', '_coerce_', '_coerce_c', '_coerce_impl', '_coerce_self', '_coerce_try', '_gens', '_gens_dict', '_has_coerce_map_from', '_ideal_class_', '_latex_names', '_list', '_one_element', '_pseudo_fraction_field', '_random_nonzero_element', '_richcmp', '_unit_ideal', '_zero_element', '_zero_ideal', 'algebraic_closure', 'base_extend', 'characteristic', 'class_group', 'coerce_map_from_c', 'coerce_map_from_impl', 'content', 'divides', 'extension', 'fraction_field', 'gcd', 'gen', 'gens', 'get_action_c', 'get_action_impl', 'has_coerce_map_from_c', 'has_coerce_map_from_impl', 'ideal', 'ideal_monoid', 'integral_closure', 'is_atomic_repr', 'is_commutative', 'is_field', 'is_finite', 'is_integral_domain', 'is_integrally_closed', 'is_noetherian', 'is_prime_field', 'is_ring', 'is_subring', 'is_zero', 'krull_dimension', 'list', 'ngens', 'one', 'one_element', 'order', 'prime_subfield', 'principal_ideal', 'quo', 'quotient', 'quotient_ring', 'random_element', 'unit_ideal', 'zero', 'zero_element', 'zero_ideal', 'zeta', 'zeta_order']"}︡
︠8a125cd3-c38a-4893-bf14-edad4f43cc18i︠
%html
<p>Any ring in Sage has a so-called base and a base ring (sometimes the base ring, the base and the ring itself coincide). In our implementation of fraction fields, we use R as base of the fraction field of R, and we use the base ring of R as the base ring of its fraction field. The usual implementation of fraction fields in Sage does the same:</p>

︡6e524427-d819-4f5c-b69b-b4e83df0642d︡{"html": "<p>Any ring in Sage has a so-called base and a base ring (sometimes the base ring, the base and the ring itself coincide). In our implementation of fraction fields, we use R as base of the fraction field of R, and we use the base ring of R as the base ring of its fraction field. The usual implementation of fraction fields in Sage does the same:</p>"}︡
︠9f39cf41-778b-41ec-97df-f3880338ce0a︠
Frac(QQ['x']).base(), Frac(QQ['x']).base_ring()
︡1babb15b-43ce-404e-bc0e-067f67ac4891︡{"stdout": "(Univariate Polynomial Ring in x over Rational Field, Rational Field)"}︡
︠6ffeee87-33ef-410e-86f1-65cd508c49aai︠
%html
<p>Note that some rings coincide with their base. And also note that not all quotient fields in Sage use base() to tell what ring they were obtained from. A notorious exception is the rational field:</p>

︡d9e8b165-45c5-490a-8948-07fc038863cf︡{"html": "<p>Note that some rings coincide with their base. And also note that not all quotient fields in Sage use base() to tell what ring they were obtained from. A notorious exception is the rational field:</p>"}︡
︠57f19c62-d99c-425e-9ee1-5da334abbae9︠
QQ.base() is QQ is QQ.base_ring()
︡5b7d79f6-3364-4165-8b36-1dfc7084afa6︡{"stdout": "True"}︡
︠62ff2140-4bfa-449c-8fe8-0b546f7cef68i︠
%html
<p>It is very easy to declare a ring as the base of a field: One just passes the ring as argument to the field constructor.</p>

︡95878d24-ac7c-4b57-a042-f16edcc8d883︡{"html": "<p>It is very easy to declare a ring as the base of a field: One just passes the ring as argument to the field constructor.</p>"}︡
︠321a9c9b-c5d3-4837-857f-76b09a39e517︠
Field(ZZ['x']).base()
︡5fb7e946-dd75-4322-9e22-c8aff94557d8︡{"stdout": "Univariate Polynomial Ring in x over Integer Ring"}︡
︠896d422e-b0d7-41dc-b05a-8b1bd1527295i︠
%html
<p>Hence, in our first approach, we simply write a class inheriting from Field, and can use the default initialisation. We provide our class with a custom string representation, such that it prints like a fraction field. In Python, the string representation is given by a method __repr__. Sage has a default implementation of __repr__, that relies on another method: _repr_, with single underscore. Hence, <strong>one implements _repr_, not __repr__. </strong>We have already mentioned the base_ring method, and a method returning&nbsp; the characteristic of the field is implicitly used in an example below.</p>

︡0f29fb00-130e-459a-a0ac-4c015f28e9de︡{"html": "<p>Hence, in our first approach, we simply write a class inheriting from Field, and can use the default initialisation. We provide our class with a custom string representation, such that it prints like a fraction field. In Python, the string representation is given by a method __repr__. Sage has a default implementation of __repr__, that relies on another method: _repr_, with single underscore. Hence, <strong>one implements _repr_, not __repr__. </strong>We have already mentioned the base_ring method, and a method returning&nbsp; the characteristic of the field is implicitly used in an example below.</p>"}︡
︠fc1e77ef-a1fd-4ce4-8544-648a9028aca8︠
class MyFrac(Field):
    def _repr_(self):
        return "NewFrac(%s)"%repr(self.base())
    def base_ring(self):
        return self.base().base_ring()
    def characteristic(self):
        return self.base().characteristic()
︡f75b41a6-b9f5-4461-a002-877ad833f83e︡︡
︠2f12635a-5203-4d52-bd50-bfe4e4f1f495︠
MyFrac(ZZ['x'])
︡92def8fb-a0a9-4af8-a04d-6c314f7a131e︡{"stdout": "NewFrac(Univariate Polynomial Ring in x over Integer Ring)"}︡
︠2df51219-1696-49be-85bc-61fcc8525acdi︠
%html
<p>Note that some parts of our example depend on fixes that are provided in trac tickets #9944 and #9138. We test whether they are merged into the Sage version you are using, such that we can work around, if needed.</p>

︡5592f992-8f5c-48ab-9961-0389d605ef8b︡{"html": "<p>Note that some parts of our example depend on fixes that are provided in trac tickets #9944 and #9138. We test whether they are merged into the Sage version you are using, such that we can work around, if needed.</p>"}︡
︠d1bdc38d-5a53-4957-ab53-541c92528b59︠
try:
    Field('bla')
    is_9138_merged = False
except:
    is_9138_merged = (ZZ['x'] in UniqueFactorizationDomains())
is_9138_merged
︡17e7097f-8112-4b92-b4a9-b9f69e23226d︡{"stdout": "False"}︡
︠25a0b1b9-7ce5-434c-8080-a8e35e8eec16i︠
%html
<p>It is automatically tested whether the base ring is actually a ring  (if  #9138 is merged). But it is not tested whether it is  a unique  factorisation domain; we will see to that later.</p>

︡f8536367-2f16-47ee-a26d-59614134bbc4︡{"html": "<p>It is automatically tested whether the base ring is actually a ring  (if  #9138 is merged). But it is not tested whether it is  a unique  factorisation domain; we will see to that later.</p>"}︡
︠245cb3a4-226a-4969-b32a-1a6672c2b850︠
MyFrac('bla') # it should raise an error, if #9138 is merged
︡00754834-4d51-42ff-a97c-05569ef110c6︡{"stdout": "NewFrac('bla')"}︡
︠eaea99d8-e913-4938-9154-284382c4363c︠
MyFrac(Integers(15))
︡59d3815b-d544-4307-9b27-7d9a8dafc6b9︡{"stdout": "NewFrac(Ring of integers modulo 15)"}︡
︠087452ec-c09e-4854-be91-469fa897b7f9i︠
%html
<h3>The elements</h3>
<p>We use the base class :class:`sage.structure.element.FieldElement`. Note that field elements do not require that their parent is a field:</p>

︡9d744dd1-aefc-41a5-a958-df94e4883dcf︡{"html": "<h3>The elements</h3>\n<p>We use the base class :class:`sage.structure.element.FieldElement`. Note that field elements do not require that their parent is a field:</p>"}︡
︠e81b92a8-b4d6-4c97-8b38-4abfee634f32︠
from sage.structure.element import FieldElement
︡622ce6c5-35cb-48e1-9457-9b09188a49ca︡︡
︠e1713eeb-b565-406c-bb3a-6b2d955d3441︠
FieldElement(ZZ)
︡22090e33-c091-4986-aa8b-3c82a171337f︡{"stdout": "Generic element of a structure"}︡
︠879bf08d-ddc0-4cd7-ae3a-6f40c41aa642i︠
%html
<p>An element of a fraction field is determined by a numerator and a denominator, which both live in the base ring of the fraction field, and the denominator must not be zero. We take care of that during initialisation, where we also normalise the denominator. By default, the denominator is one.</p>
<p>We provide methods that return numerator and denominator, and we provide a string representation that differs from the usual representation of fraction field elements in Sage, in order to see more easily whether an example uses the existing fraction fields or our new implementation.</p>

︡ab65cc23-1d6f-4178-a46a-b0d4e5d0767f︡{"html": "<p>An element of a fraction field is determined by a numerator and a denominator, which both live in the base ring of the fraction field, and the denominator must not be zero. We take care of that during initialisation, where we also normalise the denominator. By default, the denominator is one.</p>\n<p>We provide methods that return numerator and denominator, and we provide a string representation that differs from the usual representation of fraction field elements in Sage, in order to see more easily whether an example uses the existing fraction fields or our new implementation.</p>"}︡
︠ed9243b7-eb90-4652-a9a9-e35533dde526︠
class MyElement(FieldElement):
    def __init__(self, n,d=None, parent=None):
        if parent is None:
            raise ValueError, "The parent must be provided"
        B = parent.base()
        if d is None:
            d = B.one_element()
        if n not in B or d not in B:
            raise ValueError, "Numerator and denominator must be elements of %s"%B
        if d==0:
            raise ZeroDivisionError, "The denominator must not be zero"
        if d<0:
            self.n = -n
            self.d = -d
        else:
            self.n = n
            self.d = d
        FieldElement.__init__(self,parent)
    def numerator(self):
        return self.n
    def denominator(self):
        return self.d
    def _repr_(self):
        return "(%s):(%s)"%(self.n,self.d)
︡d698c39c-2230-4ae2-b279-02e7e6bfc6f6︡︡
︠d0a3f20d-33e2-4d60-ac68-75dc4c7b8147︠
MyElement(4,-3,ZZ)
︡a1e7e6e3-e0e1-488a-8e79-012589b656e5︡{"stdout": "(-4):(3)"}︡
︠ccb3ea62-fcba-4159-9398-66ebdd82913f︠
MyElement(4,-3.1, ZZ)
︡f068ea12-7d55-44d5-beb9-94b4254c3993︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_17.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"TXlFbGVtZW50KDQsLTMuMSwgWlop\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpfrjl3W/___code___.py\", line 3, in <module>\n    exec compile(u'MyElement(_sage_const_4 ,-_sage_const_3p1 , ZZ)' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"\", line 9, in __init__\n    \nValueError: Numerator and denominator must be elements of Integer Ring"}︡
︠4a991787-79b6-413d-b973-9af364836b02︠
MyElement(4, 0, ZZ)
︡949813cb-ac0d-4c72-87df-76cf8562998c︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_18.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"TXlFbGVtZW50KDQsIDAsIFpaKQ==\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmp1Bb6eG/___code___.py\", line 3, in <module>\n    exec compile(u'MyElement(_sage_const_4 , _sage_const_0 , ZZ)' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"\", line 11, in __init__\n    \nZeroDivisionError: The denominator must not be zero"}︡
︠a38f1357-27ea-491d-b1ab-ec896f2ff8cc︠
MyElement(3, parent=ZZ)
︡0983cdc7-bdd8-4d4e-abc4-45b76c121db0︡{"stdout": "(3):(1)"}︡
︠b82edbf0-ec1f-4b89-9ba9-7bcf4bbd9423i︠
%html
<h2>Implementing arithmetic operations</h2>
<p>In Python and Cython, arithmetic operations are implemented using double underscore methods such as __add__, __mul__, __div__, __invert__. Again, for most of them, Sage provides a default implementation that makes use of a single underscore method to be provided by the user. The default implementation uses Sage's powerful coercion model (see below). Therefore, <strong>one must not override the default arithmetic methods. </strong></p>
<p>Before a single underscore method is called, the default implementation of the double underscore methods makes sure that all arguments belong to the same parent. So, when implementing the single underscore methods, we can assume that both arguments belong to the same parent.</p>
<p>Let us now refine our element class, so that we can do some arithmetic. Note that when constructing a new element, we do <em>not </em>directly name our class MyElement. Instead, we return <em>an instance of the same class as self</em>. The reason is that later we will in fact work with sub-classes of MyElement.</p>
<p>We also implement comparison with the method __cmp__, where we can again assume that both arguments belong to the same parent. <em>Note that if you program in Cython, you must use a slightly different way, that is explained in the module sage.structure.element.</em></p>

︡54a97c80-c092-4ab3-b42d-950df5e6885a︡{"html": "<h2>Implementing arithmetic operations</h2>\n<p>In Python and Cython, arithmetic operations are implemented using double underscore methods such as __add__, __mul__, __div__, __invert__. Again, for most of them, Sage provides a default implementation that makes use of a single underscore method to be provided by the user. The default implementation uses Sage's powerful coercion model (see below). Therefore, <strong>one must not override the default arithmetic methods. </strong></p>\n<p>Before a single underscore method is called, the default implementation of the double underscore methods makes sure that all arguments belong to the same parent. So, when implementing the single underscore methods, we can assume that both arguments belong to the same parent.</p>\n<p>Let us now refine our element class, so that we can do some arithmetic. Note that when constructing a new element, we do <em>not </em>directly name our class MyElement. Instead, we return <em>an instance of the same class as self</em>. The reason is that later we will in fact work with sub-classes of MyElement.</p>\n<p>We also implement comparison with the method __cmp__, where we can again assume that both arguments belong to the same parent. <em>Note that if you program in Cython, you must use a slightly different way, that is explained in the module sage.structure.element.</em></p>"}︡
︠795f6dc5-0bbc-4977-960f-486a79b80962︠
class MyElement(MyElement):
    def __cmp__(self, other):
        return cmp(self.n*other.denominator(), other.numerator()*self.d)
    def _add_(self, other):
        C = self.__class__
        D = self.d*other.denominator()
        return C(self.n*other.denominator()+self.d*other.numerator(),D, self.parent())
    def _sub_(self, other):
        C = self.__class__
        D = self.d*other.denominator()
        return C(self.n*other.denominator()-self.d*other.numerator(),D, self.parent())
    def _mul_(self, other):
        C = self.__class__
        return C(self.n*other.numerator(), self.d*other.denominator(), self.parent())
    def _div_(self, other):
        C = self.__class__
        return C(self.n*other.denominator(), self.d*other.numerator(), self.parent())
︡73d723bc-61fe-4757-b0dd-c3339f3e122a︡︡
︠b287ed03-7439-4ed4-acba-00c4be246403︠
P = MyFrac(ZZ)
a = MyElement(3,4,P)
b = MyElement(1,2,P)
︡a44b2cc7-c08b-4107-ad29-553e30a6c7ce︡︡
︠479dacf7-9417-454e-8b82-2ab04d812574︠
a+b, a-b, a*b, a/b
︡eb63370a-adcf-4d63-9d46-9b589523ea5d︡{"stdout": "((10):(8), (2):(8), (3):(8), (6):(4))"}︡
︠d0223664-5ae6-4fe1-b759-e1f42eef4d72i︠
%html
<p>Note that there is a default implementation of exponentiation. Hence, unless you want to provide a more efficient way, you can rely on the default:</p>

︡4f8a393f-7a19-4703-ab98-a84ca6fdadb0︡{"html": "<p>Note that there is a default implementation of exponentiation. Hence, unless you want to provide a more efficient way, you can rely on the default:</p>"}︡
︠98237cfa-3432-44b7-a14f-6325722df6b7︠
a^3
︡15905671-3774-4478-9810-07b7a6b73d2c︡{"stdout": "(27):(64)"}︡
︠c84d2ad6-5259-45ba-9cff-c5a3a115c787i︠
%html
<p>We verify that the comparison does as we wish:</p>

︡0edd709e-7b66-4599-bd86-c8c222139df0︡{"html": "<p>We verify that the comparison does as we wish:</p>"}︡
︠a86c530b-9c04-4266-b077-e784dab09061︠
a-b == MyElement(1,4,P)
︡b79520fa-6476-4d14-bf7f-1e1d47b72d47︡{"stdout": "True"}︡
︠894e35cb-9834-447c-8ca9-cf8b96b240bci︠
%html
<p>Also note that there is a default implementation of element tests. So, we can do</p>

︡275f300d-be41-4b9d-92cf-c9f6ac5bec31︡{"html": "<p>Also note that there is a default implementation of element tests. So, we can do</p>"}︡
︠8d812e49-8fa8-4a6f-ac3d-9bb0b3c0f59f︠
a in P
︡c6c49f57-8689-4bef-92f2-838a8e51be86︡{"stdout": "True"}︡
︠5f5000a3-77bc-4c3f-8853-97e34b2eaca6i︠
%html
<p>but we can, for example, not verify that the integers are contained in the fraction field of the ring of integers:</p>

︡6469d465-bb9a-4d3f-b283-3d1e077860fa︡{"html": "<p>but we can, for example, not verify that the integers are contained in the fraction field of the ring of integers:</p>"}︡
︠3e80769d-2b73-4bd3-9ebb-b88388f394dc︠
1 in P
︡8ffe6a1e-a516-4d97-955c-b19cd3599675︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_26.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"MSBpbiBQ\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpUIbdaA/___code___.py\", line 3, in <module>\n    exec compile(u'_sage_const_1  in P' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"parent.pyx\", line 939, in sage.structure.parent.Parent.__contains__ (sage/structure/parent.c:6635)\n  File \"parent.pyx\", line 862, in sage.structure.parent.Parent.__call__ (sage/structure/parent.c:6283)\nNotImplementedError"}︡
︠831fc2ae-ebee-4ff7-b237-6c7066ff5a39i︠
%html
<p>We will take care of that later.</p>
<h2>Chosing a category</h2>
<p>As we have mentioned above, a parent can be understood as an object of a category (in the sense of algebra). It is no surprise that our parent P defined above knows that it belongs to the category of fields, as it is derived from the base class of fields.</p>

︡d9b2f53a-c647-4997-a7c8-0167fd842d9a︡{"html": "<p>We will take care of that later.</p>\n<h2>Chosing a category</h2>\n<p>As we have mentioned above, a parent can be understood as an object of a category (in the sense of algebra). It is no surprise that our parent P defined above knows that it belongs to the category of fields, as it is derived from the base class of fields.</p>"}︡
︠08933aec-3763-4cb7-969b-118821822097︠
P.category()
︡0b057acc-2a6a-45c3-babe-01f0e8fdbdcf︡{"stdout": "Category of fields"}︡
︠59065864-3190-4cca-90d7-206c82ac1fdai︠
%html
<p>However, we could choose a smaller category, namely the category of quotient fields.</p>
<h3>Why should one choose a category?</h3>
<p>As usual in Python, one can define base classes, and then all classes derived from them inherit certain default methods; we have seen examples above. In addition to that, one can provide default methods <em>for all objects of a category</em>, and <em>for all elements of such objects</em>. In other words, there is a different way of inheriting useful stuff. It does not rely on implementation (such as class inheritance), but it relies on mathematical concepts.</p>
<p>In addition, the categories define test suites for their objects and elements. Hence, one also gets basic sanity tests for free.</p>
<p>How does this so-called <em>category framework </em>work?</p>
<p>Any category defines a base class for their objects ("parent class") and of the elements of their objects ("element class"). If the category is declared during initialisation of a parent, then the class of that parent is automatically <em>dynamically changed</em> into a sub-class of the category's parent class. Hence, it inherits additional methods. Note that dynamic classes do not work in Cython. Nevertheless, if one writes a parent in Cython, it still inherits the methods from the category: It is not by class inheritance but by virtue of a default implementation of __getattr__ that mimmicks the class inheritance.</p>
<p><strong>So, it is strongly recommended to use the category framework both in Python and in Cython.</strong></p>
<p>Let us see whether there is any gain in chosing the category of quotient fields instead of the category of fields:</p>

︡f608b5ff-ec2e-4705-a50d-b17af6b744e8︡{"html": "<p>However, we could choose a smaller category, namely the category of quotient fields.</p>\n<h3>Why should one choose a category?</h3>\n<p>As usual in Python, one can define base classes, and then all classes derived from them inherit certain default methods; we have seen examples above. In addition to that, one can provide default methods <em>for all objects of a category</em>, and <em>for all elements of such objects</em>. In other words, there is a different way of inheriting useful stuff. It does not rely on implementation (such as class inheritance), but it relies on mathematical concepts.</p>\n<p>In addition, the categories define test suites for their objects and elements. Hence, one also gets basic sanity tests for free.</p>\n<p>How does this so-called <em>category framework </em>work?</p>\n<p>Any category defines a base class for their objects (\"parent class\") and of the elements of their objects (\"element class\"). If the category is declared during initialisation of a parent, then the class of that parent is automatically <em>dynamically changed</em> into a sub-class of the category's parent class. Hence, it inherits additional methods. Note that dynamic classes do not work in Cython. Nevertheless, if one writes a parent in Cython, it still inherits the methods from the category: It is not by class inheritance but by virtue of a default implementation of __getattr__ that mimmicks the class inheritance.</p>\n<p><strong>So, it is strongly recommended to use the category framework both in Python and in Cython.</strong></p>\n<p>Let us see whether there is any gain in chosing the category of quotient fields instead of the category of fields:</p>"}︡
︠487c1a68-53e8-4e30-a3d9-408750397904︠
QuotientFields().parent_class, QuotientFields().element_class
︡65ad28fe-1f9b-4c15-960f-ad5d159b3a3f︡{"stdout": "(<class 'sage.categories.quotient_fields.QuotientFields.parent_class'>, <class 'sage.categories.quotient_fields.QuotientFields.element_class'>)"}︡
︠97009651-7da3-4812-93ec-98e9a5fcd7e5︠
[p for p in dir(QuotientFields().parent_class) if p not in dir(Fields().parent_class)]; [p for p in dir(QuotientFields().element_class) if p not in dir(Fields().element_class)]
︡1b9a4dc4-6398-4cdc-a158-0ad6f098fcab︡{"stdout": "[]\n['_derivative', 'denominator', 'derivative', 'factor', 'numerator', 'partial_fraction_decomposition']"}︡
︠21037160-4889-4514-85e0-b0bae0d9e731i︠
%html
<p>So, there is no immediate gain for the parent, but there is more structure for the elements.</p>
<h3>Category framework for the parent</h3>
<p>Let us start by modifying the parent's initialisation, such that the correct category is chosen resp. the correct category can be provided by the user.</p>

︡668886ef-5368-4eec-a4e3-49dd52c234e8︡{"html": "<p>So, there is no immediate gain for the parent, but there is more structure for the elements.</p>\n<h3>Category framework for the parent</h3>\n<p>Let us start by modifying the parent's initialisation, such that the correct category is chosen resp. the correct category can be provided by the user.</p>"}︡
︠6674ff4e-889a-4598-ba0a-13175e38db5e︠
from sage.categories.quotient_fields import QuotientFields
class MyFrac(MyFrac):
    def __init__(self, base, category=None):
        Field.__init__(self, base, category=category or QuotientFields())
︡d2be4b21-fa14-4eca-9a17-7c0d1cd97073︡︡
︠40be5f35-aad3-4d90-b41a-ceb866572e4ci︠
%html
<p>We demonstrate that by declaring the category, when constructing instances of MyFrac, their class is dynamically changed into a new class MyFrac_with_category. It is a common sub-class of MyFrac and of the category's parent class.</p>

︡76a75c69-8833-4c9b-b351-969fa4a68a31︡{"html": "<p>We demonstrate that by declaring the category, when constructing instances of MyFrac, their class is dynamically changed into a new class MyFrac_with_category. It is a common sub-class of MyFrac and of the category's parent class.</p>"}︡
︠f9797fd7-9c67-4026-8c31-94e7ccdc83ee︠
P = MyFrac(ZZ)
type(P); isinstance(P,MyFrac); isinstance(P,QuotientFields().parent_class)
︡b5a166f3-e7e4-4984-8b2d-4ba6e19d124c︡{"stdout": "<class '__main__.MyFrac_with_category'>\nTrue\nTrue"}︡
︠4bbdd43b-db9c-4d2e-93c2-e0f0e6939d8di︠
%html
<p>We can now demonstrate that P inherits additional methods. The base class Field does not have a method `sum`. But the parent class of the category of fields does have such method (in fact, it is defined for the category of commutative additive monoids), and it is available for P:</p>

︡0d7341d2-bfde-4696-b3d4-b1d58258b3a6︡{"html": "<p>We can now demonstrate that P inherits additional methods. The base class Field does not have a method `sum`. But the parent class of the category of fields does have such method (in fact, it is defined for the category of commutative additive monoids), and it is available for P:</p>"}︡
︠a9b7f813-763e-4ca9-b8e0-6fcbfe1fa250︠
a = MyElement(3,4,P)
b = MyElement(1,2,P)
c = MyElement(-1,2,P)
P.sum.__module__
︡a9b5e100-0acd-4104-a312-ebf1aabff73d︡{"stdout": "'sage.categories.commutative_additive_monoids'"}︡
︠4ec9be54-7040-4ca6-a281-b102358010f7i︠
%html
<p>However, it does not work, yet, because some other stuff is not properly implemented:</p>

︡50d3f805-ff96-4332-a92a-0d2321ca4910︡{"html": "<p>However, it does not work, yet, because some other stuff is not properly implemented:</p>"}︡
︠44d5a1fb-834d-4524-84fc-201dc83b523e︠
P.sum([a,b,c])
︡29e5241b-a04b-43ef-8518-93f76e910261︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_33.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"UC5zdW0oW2EsYixjXSk=\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpJh3f8_/___code___.py\", line 2, in <module>\n    exec compile(u'P.sum([a,b,c])' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/usr/local/sage2/local/lib/python2.6/site-packages/sage/categories/commutative_additive_monoids.py\", line 137, in sum\n    return sum(args, self.zero())\n  File \"ring.pyx\", line 502, in sage.rings.ring.Ring.zero_element (sage/rings/ring.c:4989)\n  File \"parent.pyx\", line 862, in sage.structure.parent.Parent.__call__ (sage/structure/parent.c:6283)\nNotImplementedError"}︡
︠51ee29a4-eb65-4817-b420-bb7359c7a05di︠
%html
<h3>Category framework for the elements</h3>
<p>The basic idea remains the same: A Python class is dynamically changed into a sub-class of the element class of a category, and that behaviour is mimmicked for Cython classes as well. However, the category framework is invoked in a different way for elements than for parents.</p>
<p>One has to provide the parent with an attribute called "Element", whose value is the class that is used to implement all elements of that parent. Then, there is a :class:`~sage.misc.lazy_attribute.lazy_attribute` of the name "element_class", that automatically mixes the class provided by "Element" with the element class of the parent's category. Note that this holds if the parent class is written in Python. For parents written in Cython, lazy attributes will only be available if trac ticket #11115 is merged.</p>
<p><strong>For providing our fraction fields with their own element classes, we just need to add a single line to our class:</strong></p>

︡cbdee8f1-866b-4732-aae7-806ea2406d41︡{"html": "<h3>Category framework for the elements</h3>\n<p>The basic idea remains the same: A Python class is dynamically changed into a sub-class of the element class of a category, and that behaviour is mimmicked for Cython classes as well. However, the category framework is invoked in a different way for elements than for parents.</p>\n<p>One has to provide the parent with an attribute called \"Element\", whose value is the class that is used to implement all elements of that parent. Then, there is a :class:`~sage.misc.lazy_attribute.lazy_attribute` of the name \"element_class\", that automatically mixes the class provided by \"Element\" with the element class of the parent's category. Note that this holds if the parent class is written in Python. For parents written in Cython, lazy attributes will only be available if trac ticket #11115 is merged.</p>\n<p><strong>For providing our fraction fields with their own element classes, we just need to add a single line to our class:</strong></p>"}︡
︠51a385ff-4d4d-47f2-99f2-1328e4448227︠
class MyFrac(MyFrac):
    Element = MyElement
︡dc3035be-66b8-4c50-9093-6d367e63f9c4︡︡
︠7f401b9b-5f95-4420-b08d-4568f473365bi︠
%html
<p>That little change has dramatic consequences. We can now create elements by simply calling the parent, we can suddenly use a method zero_element, and the "sum" method works:</p>

︡277d15ae-c450-4c4c-b42f-a4f4ad381487︡{"html": "<p>That little change has dramatic consequences. We can now create elements by simply calling the parent, we can suddenly use a method zero_element, and the \"sum\" method works:</p>"}︡
︠d7736519-1fbc-422b-bff9-b2fde2236433︠
P = MyFrac(ZZ)
︡6d9b6308-d884-4ea2-a8f5-4287a2414c1d︡︡
︠6e42895c-a291-4ce2-9fe8-9873a4fa9e09︠
P(1), P(2,3)
︡85857699-4bb4-411f-a1bf-55bd6ac1ff40︡{"stdout": "((1):(1), (2):(3))"}︡
︠1102acdd-cd43-40fd-a8c7-61b8c4c3599e︠
P.zero_element()
︡e8f020a5-b64e-4e84-a6c6-493bc0d5da2d︡{"stdout": "(0):(1)"}︡
︠a97806ca-e379-4a5c-a1ff-2509ad0f1b06︠
a = MyElement(9,4,P)
b = MyElement(1,2,P)
c = MyElement(-1,2,P)
︡9652acd0-421f-4e27-b5d9-96be81c15b36︡︡
︠ff4e5e96-edd6-4677-8c97-b35e10e5f27d︠
P.sum([a,b,c])
︡2dd15fac-9d84-407e-ab14-f0965c345393︡{"stdout": "(36):(16)"}︡
︠889c726c-56ec-4319-a51a-d7b99a3303d1i︠
%html
<p>How did that happen?</p>
<p>Since we provided the attribute "Element", P has now also an attribute "element_class", that is a sub-class of both MyElement and of `P.category().element_class`:</p>

︡43edbc73-61b2-41ef-9db4-bc866315bff4︡{"html": "<p>How did that happen?</p>\n<p>Since we provided the attribute \"Element\", P has now also an attribute \"element_class\", that is a sub-class of both MyElement and of `P.category().element_class`:</p>"}︡
︠cff36e70-59c9-496c-9543-0c6b568c72aa︠
P.element_class; issubclass(P.element_class, (MyElement,P.category().element_class))
︡bbadceba-2f0b-416e-bb1e-6df3c0f515a5︡{"stdout": "<class '__main__.MyFrac_with_category.element_class'>\nTrue"}︡
︠e98b5b0d-81bf-4362-ba1b-e4b9cbf7ea07i︠
%html
<p>The default __call__ method of a parent P simply passes the arguments to P.element_class, adding a named argument "parent=P". That is the reason for the choice of argument names in MyElement.__init__, and for their order.</p>
<p>In particular, elements obtained by calling P are instances of that new automatically created class.</p>

︡ee1e6cdd-c186-490c-92c1-f0bdf422bf64︡{"html": "<p>The default __call__ method of a parent P simply passes the arguments to P.element_class, adding a named argument \"parent=P\". That is the reason for the choice of argument names in MyElement.__init__, and for their order.</p>\n<p>In particular, elements obtained by calling P are instances of that new automatically created class.</p>"}︡
︠985529c6-9f7d-4b25-bcb5-1bb9adbe964f︠
type(P(2,3))
︡f19863d0-466a-48bc-b386-2994427d7e89︡{"stdout": "<class '__main__.MyFrac_with_category.element_class'>"}︡
︠b9f0d3c3-95f3-434c-9aae-3ecde6192cc0i︠
%html
<p>That is the reason why we did not explicitly name the class in the arithmetic methods of MyElement. If we create one element by calling P (this is the case, e.g., for P.zero_element()), and add any other instance of MyElement to it, then the sum will also be an instance of P.element_class. Since P.zero_element() is an instance of P.element_class and since P.sum([...]) starts with P.zero_element(), we thus have:</p>

︡f33dd59b-6873-442e-9a77-b159acb3dff4︡{"html": "<p>That is the reason why we did not explicitly name the class in the arithmetic methods of MyElement. If we create one element by calling P (this is the case, e.g., for P.zero_element()), and add any other instance of MyElement to it, then the sum will also be an instance of P.element_class. Since P.zero_element() is an instance of P.element_class and since P.sum([...]) starts with P.zero_element(), we thus have:</p>"}︡
︠db56193e-526c-4453-b4de-31f22f33784f︠
type(a); isinstance(a,P.element_class); type(P.sum([a,b,c]))
︡a53cf3f2-8325-4be6-bd76-cc95cb1f4bf9︡{"stdout": "<class '__main__.MyElement'>\nFalse\n<class '__main__.MyFrac_with_category.element_class'>"}︡
︠3397793b-18bb-4da4-9cb6-26a7e769d017i︠
%html
<p>We have already seen above that the element class of P.category() provides a method factor(). Let us try if it works:</p>

︡f687ae09-845d-4b50-9cf6-13f765b9e7d5︡{"html": "<p>We have already seen above that the element class of P.category() provides a method factor(). Let us try if it works:</p>"}︡
︠b05f8b6b-4f4e-4ab3-bbd0-6c90f2224948︠
a; a.factor()
︡54891722-8b42-4204-8376-1668f84f8342︡{"stdout": "(9):(4)\n2^-2 * 3^2"}︡
︠0da2dc43-371a-47c9-89db-d2241c73b075i︠
%html
<p>It does! That may come as a surprise, since a is actually not an instance of P.element_class, and its class does not know about the factor method. But, as we have mentioned earlier, the __getattr__ method works around a missing class inheritance.</p>

︡7e1f64c7-dfa1-41d7-ab8c-feff07e6fa01︡{"html": "<p>It does! That may come as a surprise, since a is actually not an instance of P.element_class, and its class does not know about the factor method. But, as we have mentioned earlier, the __getattr__ method works around a missing class inheritance.</p>"}︡
︠e871e86a-94d8-4636-afba-ea079222c85e︠
hasattr(type(a), 'factor'); hasattr(P.element_class, 'factor'); hasattr(a, 'factor')
︡fbc8069f-3d7c-4c07-990d-d874bd6011c1︡{"stdout": "False\nTrue\nTrue"}︡
︠9d21f327-73f4-4715-a04b-45dd4dc60d60i︠
%html
<p>Note, however, that the access to the factor method is much faster if one has an actual instance of P.element_class:</p>

︡8f83b999-5dba-496c-bd59-8febdc088741︡{"html": "<p>Note, however, that the access to the factor method is much faster if one has an actual instance of P.element_class:</p>"}︡
︠aea39300-df06-431b-9d67-b1c1d57b8b44︠
timeit('a.factor')
︡4cec3948-18b9-43e7-9ba7-a1be41c90e52︡{"stdout": "625 loops, best of 3: 8.36 \u00b5s per loop"}︡
︠cdad4aca-5d67-4e93-9560-82b03df8fb6f︠
a2 = P(9,4)
a2 == a
︡69f6cd8c-87ee-40a0-bf5b-f7ae58d611c1︡{"stdout": "True"}︡
︠c48c4219-c5f5-4fb3-9ccc-bdf2d08a55d0︠
timeit('a2.factor')
︡768d5528-2964-423e-8fac-7cade63d660f︡{"stdout": "625 loops, best of 3: 303 ns per loop"}︡
︠149ad543-463e-4864-bf21-76e61ac692eai︠
%html
<h2>Uniqueness of parents</h2>
<p>In sage, all parents should be globally unique. That means: If two parents are equal then they should actually be the same objects - even if their definition looks different. For example:</p>

︡5eef9d7d-9115-4077-b2f9-32910255612b︡{"html": "<h2>Uniqueness of parents</h2>\n<p>In sage, all parents should be globally unique. That means: If two parents are equal then they should actually be the same objects - even if their definition looks different. For example:</p>"}︡
︠b32f8eb0-d474-4404-9a67-19cae13addba︠
QQ['x'] is PolynomialRing(QQ,names=['x'])
︡d906741a-e83a-485d-9453-eea954a906f4︡{"stdout": "True"}︡
︠da155014-dc3c-4e0e-9028-62979d178209︠
Frac(QQ['x']) is PolynomialRing(QQ,names=['x']).fraction_field()
︡16a27b6c-24d2-42f0-8dd4-abf233f5fbf6︡{"stdout": "True"}︡
︠d6e65ce9-9aa5-4a26-9823-0675c44ed4c8i︠
%html
<p>That requirement is not met by our implementation, yet:</p>

︡eb8a92bb-425e-4b13-a676-67e078c40a77︡{"html": "<p>That requirement is not met by our implementation, yet:</p>"}︡
︠0d98c9d7-140a-4bb6-864a-ccdde84ff7b6︠
MyFrac(ZZ) is MyFrac(ZZ)
︡a48f9d25-184f-4ba3-bda4-be0b29c93c41︡{"stdout": "False"}︡
︠83845ffe-c6b0-4ef9-8380-d8a3884445a7i︠
%html
<p>There are several tools in Sage to construct unique parents. See, for instance, :class:`sage.structure.factory.UniqueFactory`, which can be used for rather complicated cases, where very different input may result in the same objects.</p>
<p>Here, we have a simpler case, and we use Python, so that we can use :class:`sage.structure.unique_representation.UniqueRepresentation`. The simplemost usage would be to make MyFrac inherit first from UniqueRepresentation and then from sage.rings.field.Field.</p>
<p>However, as we have announced above, we want that our fraction fields test whether the given ring is a unique factorisation domain. Therefore, we want to preprocess the arguments before passing them to the __init__ method. That can be done in a __classcall__ method, that needs to be a static method.</p>
<p>Note that testing whether a ring is a unique factorisation domain requires trac ticket #9138 being merged - without that, even the ring of integers would not known as a unique factorisation domain (even though it is known as Euclidean domain). Anyway, we make our test depend on the variable is_9138_merged defined above; if the ticket is not merged, then we simply test if the input is a ring.</p>

︡05a6770f-6bd7-4482-8f86-526458ca3ef4︡{"html": "<p>There are several tools in Sage to construct unique parents. See, for instance, :class:`sage.structure.factory.UniqueFactory`, which can be used for rather complicated cases, where very different input may result in the same objects.</p>\n<p>Here, we have a simpler case, and we use Python, so that we can use :class:`sage.structure.unique_representation.UniqueRepresentation`. The simplemost usage would be to make MyFrac inherit first from UniqueRepresentation and then from sage.rings.field.Field.</p>\n<p>However, as we have announced above, we want that our fraction fields test whether the given ring is a unique factorisation domain. Therefore, we want to preprocess the arguments before passing them to the __init__ method. That can be done in a __classcall__ method, that needs to be a static method.</p>\n<p>Note that testing whether a ring is a unique factorisation domain requires trac ticket #9138 being merged - without that, even the ring of integers would not known as a unique factorisation domain (even though it is known as Euclidean domain). Anyway, we make our test depend on the variable is_9138_merged defined above; if the ticket is not merged, then we simply test if the input is a ring.</p>"}︡
︠8946f44e-f24a-496e-841c-9dca15ce7dc7︠
from sage.categories.unique_factorization_domains import UniqueFactorizationDomains
from sage.structure.unique_representation import UniqueRepresentation
class MyFracUnique(UniqueRepresentation, MyFrac):
    @staticmethod
    def __classcall__(cls, base, category=None):
        if (base not in Rings()) or (is_9138_merged and base not in UniqueFactorizationDomains()):
            raise TypeError, "%s is no unique factorisation domain"%base
        return super(MyFracUnique, cls).__classcall__(cls, base, category)
︡457c7ac2-3712-4661-a8a0-70fdf4f714e2︡︡
︠28e31b97-8451-47e2-b4dc-d98b16b9ad25i︠
%html
<p>Now, uniqueness holds, and rings that are no unique factorisation domains are refused.</p>

︡b26c722f-cb53-4f52-a7fd-1bf257de752a︡{"html": "<p>Now, uniqueness holds, and rings that are no unique factorisation domains are refused.</p>"}︡
︠7c55a81c-170a-406b-8272-42764636cb27︠
MyFracUnique(ZZ) is MyFracUnique(ZZ)
︡9c68c783-6bc4-4a17-ac96-604e85161a70︡{"stdout": "True"}︡
︠ddfd59f3-828d-43a4-8a5c-8f699c458bad︠
MyFracUnique(ZZ['x']) is MyFracUnique(ZZ['x'])
︡250d414c-5d4c-474a-b4e5-e8eff3c79477︡{"stdout": "True"}︡
︠2509c3d3-9ca3-4a86-8103-b3298042ff63︠
MyFracUnique(Integers(15)) # should raise an error if #9138 is merged
︡975f3e64-63e3-452d-93d4-ccf79fd7b435︡{"stdout": "NewFrac(Ring of integers modulo 15)"}︡
︠42a2e748-eaba-41a8-99d9-4976afb70424︠
MyFracUnique(SymmetricGroup(5)) # should raise the error even without #9138
︡268ce348-c84a-492a-8f5d-e274acfb0ac2︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_55.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"TXlGcmFjVW5pcXVlKFN5bW1ldHJpY0dyb3VwKDUpKSAjIHNob3VsZCByYWlzZSB0aGUgZXJyb3IgZXZlbiB3aXRob3V0ICM5MTM4\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpurf9bH/___code___.py\", line 3, in <module>\n    exec compile(u'MyFracUnique(SymmetricGroup(_sage_const_5 )) # should raise the error even without #9138' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/usr/local/sage2/local/lib/python2.6/site-packages/sage/misc/classcall_metaclass.py\", line 258, in __call__\n    return cls.__classcall__(cls, *args, **options)\n  File \"\", line 5, in __classcall__\n    \nTypeError: SymmetricGroup(5) is no unique factorisation domain"}︡
︠f126181e-6190-49ec-8ecc-a06741454198i︠
%html
<p>Note that Sage's tools for creating unique parents also take care of pickling (at least to some extent):</p>

︡59afc4a5-976c-4261-aaab-04cfa2ec8b88︡{"html": "<p>Note that Sage's tools for creating unique parents also take care of pickling (at least to some extent):</p>"}︡
︠f1b66ba0-e00e-4a25-9917-865b1af91616︠
loads(dumps(MyFracUnique(ZZ))) is MyFracUnique(ZZ)
︡ac839897-d420-432b-91f7-cd4a6e5bfaf3︡{"stdout": "True"}︡
︠43958b83-3091-4edd-97a7-7d92ac243c1ci︠
%html
<h2>Coercion - the basics</h2>
<p><em>Coercion is involved if one wants to be able to do arithmetic, comparisons, etc. between elements of distinct parents.</em></p>
<h3>Theoretical background</h3>
<h4>Dissociation from <em>type conversion</em></h4>
<p>Perhaps the reader is familiar with the concept of coercion in the programming language C, which is more or less a synonyme for "automatic type conversion". A very basic example of an automatic type conversion is the sum of an integer with a floating point number: The integer is automatically converted into a floating point number, and then the summation takes place.</p>
<p>However, <strong>coercion in Sage is not about a change of types, but about a change of parents.</strong> Elements of the same type may have very different parents, such as here:</p>

︡47232a1f-ab11-4198-a351-aec0f83182ea︡{"html": "<h2>Coercion - the basics</h2>\n<p><em>Coercion is involved if one wants to be able to do arithmetic, comparisons, etc. between elements of distinct parents.</em></p>\n<h3>Theoretical background</h3>\n<h4>Dissociation from <em>type conversion</em></h4>\n<p>Perhaps the reader is familiar with the concept of coercion in the programming language C, which is more or less a synonyme for \"automatic type conversion\". A very basic example of an automatic type conversion is the sum of an integer with a floating point number: The integer is automatically converted into a floating point number, and then the summation takes place.</p>\n<p>However, <strong>coercion in Sage is not about a change of types, but about a change of parents.</strong> Elements of the same type may have very different parents, such as here:</p>"}︡
︠e0060b91-51f1-446f-9a13-c0cd685e423d︠
P1 = QQ['v,w']; P2 = ZZ['v,w']
︡bc9e4986-3f92-4119-b60b-a0a8eb48a9b6︡︡
︠ad21ed51-59b8-4a7f-8c31-6977856c0deb︠
type(P1.gen()) == type(P2.gen()); P1==P2
︡6480082c-d84c-4364-9dcc-421833019f15︡{"stdout": "True\nFalse"}︡
︠297c6ba7-9e26-4c72-b685-54cd6ece8606i︠
%html
<p>Every element of P2 can be naturally interpreted as an element of P1. So, it makes sense to be able to add elements of the two rings - the result should then live in P1:</p>

︡58ba75de-415f-42a3-adde-4391b4f86d34︡{"html": "<p>Every element of P2 can be naturally interpreted as an element of P1. So, it makes sense to be able to add elements of the two rings - the result should then live in P1:</p>"}︡
︠ca1885c6-e415-4b2a-8ab2-f5f67c070565︠
(P1.gen()+P2.gen()).parent() is P1
︡959ed588-9d30-4afe-b17d-420bef506662︡{"stdout": "True"}︡
︠1d18db98-e2b3-47f2-9154-3390a1dc564ai︠
%html
<p>It would be rather inconvenient if one needed to <em>explicitly </em>convert an element of P2 into P1 before adding. The coercion system does that conversion automatically.</p>
<p>Of course, one must not overdo the automatic conversion. There are situations in which a conversion is possible, but one would not want to use that conversion for arithmetics. For example, while every element of a finite prime field can be converted into an integer (by chosing a representative), one would certainly not want to add elements of GF(2) with elements of GF(3); in particular one would not want that the result is an integer.</p>
<h4>Coercion versus conversion</h4>
<p>A coercion happens implicitly, without being explicitly requested by the user. Hence, coercion must be based on mathematical rigour. The user may want to convert a finite field element into an integer, and can do so explicitly - but such conversion must never happen implicitly. We elaborate on the conditions that a conversion needs to satisfy in order to be eligible as a coercion.</p>
<h5>A coercion is a map, a conversion is a <em>partial</em> map</h5>
<p>Let P1 and P2 be two parents. It is possible that some elements of P1 can be converted into P2, but others can't. For example, a polynomial of degree zero over the integers can be interpreted as an integer - but there is no general conversion of a polynomial into an integer.</p>
<p>The first requirement is:</p>
<p style="padding-left: 30px;">The coercion of elements of P1 into P2 must be defined <em>for all elements </em>of P1. There may be a conversion of some elements of P1 into P2, but if it is not globally defined, then it is no coercion.</p>
<p>Hence, coercion maps are defined on the level of parents, and they can be requested with the following methods:</p>

︡70ca3077-0e7a-4b0f-82f0-67e5dbf37aaa︡{"html": "<p>It would be rather inconvenient if one needed to <em>explicitly </em>convert an element of P2 into P1 before adding. The coercion system does that conversion automatically.</p>\n<p>Of course, one must not overdo the automatic conversion. There are situations in which a conversion is possible, but one would not want to use that conversion for arithmetics. For example, while every element of a finite prime field can be converted into an integer (by chosing a representative), one would certainly not want to add elements of GF(2) with elements of GF(3); in particular one would not want that the result is an integer.</p>\n<h4>Coercion versus conversion</h4>\n<p>A coercion happens implicitly, without being explicitly requested by the user. Hence, coercion must be based on mathematical rigour. The user may want to convert a finite field element into an integer, and can do so explicitly - but such conversion must never happen implicitly. We elaborate on the conditions that a conversion needs to satisfy in order to be eligible as a coercion.</p>\n<h5>A coercion is a map, a conversion is a <em>partial</em> map</h5>\n<p>Let P1 and P2 be two parents. It is possible that some elements of P1 can be converted into P2, but others can't. For example, a polynomial of degree zero over the integers can be interpreted as an integer - but there is no general conversion of a polynomial into an integer.</p>\n<p>The first requirement is:</p>\n<p style=\"padding-left: 30px;\">The coercion of elements of P1 into P2 must be defined <em>for all elements </em>of P1. There may be a conversion of some elements of P1 into P2, but if it is not globally defined, then it is no coercion.</p>\n<p>Hence, coercion maps are defined on the level of parents, and they can be requested with the following methods:</p>"}︡
︠10f22901-6408-4c3b-aab8-f3d3f3561bca︠
P1.has_coerce_map_from(P2)
︡c311e65f-4b58-4d88-b815-c80e8c1625e2︡{"stdout": "True"}︡
︠9735dc57-36f4-418d-ad73-34e660b0fc3b︠
P1.coerce_map_from(P2)
︡ab50b48e-1cc8-4610-9c99-42c37c8e3af8︡{"stdout": "Call morphism:\n  From: Multivariate Polynomial Ring in v, w over Integer Ring\n  To:   Multivariate Polynomial Ring in v, w over Rational Field"}︡
︠40cf724f-a6e0-48cd-83ff-f1b01fc76cfa︠
P2.has_coerce_map_from(P1)
︡1e537dc0-13e6-4340-9e18-bf398944ef09︡{"stdout": "False"}︡
︠a4dce37e-3e27-4d74-9037-43f20fa917ac︠
P2.coerce_map_from(P1) is None
︡6dc263b8-d88c-48a1-b8ec-ed0c0615d049︡{"stdout": "True"}︡
︠117959fe-fb79-4735-8b6c-481c5917f1e3i︠
%html
<h5>A coercion is structure preserving</h5>
<p>Any real number can be converted to an integer, namely by rounding. However, such a conversion is not useful in arithmetic operations:</p>

︡4823b871-6d91-4965-9b8a-d1ec7da8de0b︡{"html": "<h5>A coercion is structure preserving</h5>\n<p>Any real number can be converted to an integer, namely by rounding. However, such a conversion is not useful in arithmetic operations:</p>"}︡
︠189f929d-c0a2-46dd-bc0d-73180b063b6e︠
int(1.6)+int(2.7) == int(1.6+2.7)
︡1aeee0de-457d-4e56-9a98-d2e6a32dbce6︡{"stdout": "False"}︡
︠19dd8c73-4f93-4359-816e-c4d3f1e73908i︠
%html
<p>The reason is that rounding real numbers is no ring homomorphism. So, the next requirement is:</p>
<p style="padding-left: 30px;">Coercion maps are morphisms, <em>i.e.</em>, they are structure preserving.</p>
<p>It depends on the parents which structure is to be preserved. For example, if one parent is a ring and the other parent is a field, then a coercion between the two must be a ring homomorphism. The coercion from the integers into the rational field, for instance, is a homomorphism of euclidean domains:</p>

︡9daf7b77-cab9-4d71-9432-214029458136︡{"html": "<p>The reason is that rounding real numbers is no ring homomorphism. So, the next requirement is:</p>\n<p style=\"padding-left: 30px;\">Coercion maps are morphisms, <em>i.e.</em>, they are structure preserving.</p>\n<p>It depends on the parents which structure is to be preserved. For example, if one parent is a ring and the other parent is a field, then a coercion between the two must be a ring homomorphism. The coercion from the integers into the rational field, for instance, is a homomorphism of euclidean domains:</p>"}︡
︠ae843ce3-2db0-4f3c-9a2c-b01b39e35fb0︠
QQ.coerce_map_from(ZZ).category_for()
︡5bf88c79-06dc-4c61-a40c-edcc12d90d92︡{"stdout": "Category of euclidean domains"}︡
︠1fa1c64d-5877-4dd9-8c38-ccb9d954fb49i︠
%html
<h5>There is at most one coercion map between two parents</h5>
<p>There are, in general, many different homomorphisms from a ring P1 to a ring P2. It is possible that none of them is used for coercion; however, it must never occur that two different coercions are simultaneously used.</p>
<h5>Coercion maps can be composed</h5>
<p>If there is a coercion &phi; from P1 to P2 and another coercion &psi; from P2 to P3, then the composition of &phi; followed by &psi; must yield a (the) coercion from P1 to P3.</p>
<h5>The identity is a coercion</h5>
<p>There is a coercion from any parent into itself, namely the identity map. Together with the two preceding requirements, it follows: If there are conversions from P1 to P2 and from P2 to P1, then they are mutually inverse.</p>
<h4>How to fulfill the above requirements, and preserve common sense</h4>
<p>As an illustration, we discuss here the rationale behind the choice of coercions between multivariate polynomial rings in Sage. Let R be a ring.</p>
<p>Obviously, there can be many ring homomorphisms between two polynomial rings over R. So, we have to fix a canonical choice. If the polynomial ring P2 has at least as many generators as P1, then one could map generator number n of P1 to generator number n of P2. Such "place preserving" conversions would certainly qualify as a coercion, since the composition of two place preserving conversions is a place preserving conversion.</p>
<p>However, common sense would would more likely take the <em>names</em> of the generators into account: When mapping R[x,y] to R[y,x], one would rather map x to x and y to y, not the first generator x of R[x,y] to the first generator y of R[y,x].</p>
<p>Sage tries to balance common sense and generality. If two polynomial rings have the same generators (but potentially in a different order), then there is a name preserving coercion:</p>

︡51e22532-13b8-49f0-9701-52fed10df0c2︡{"html": "<h5>There is at most one coercion map between two parents</h5>\n<p>There are, in general, many different homomorphisms from a ring P1 to a ring P2. It is possible that none of them is used for coercion; however, it must never occur that two different coercions are simultaneously used.</p>\n<h5>Coercion maps can be composed</h5>\n<p>If there is a coercion &phi; from P1 to P2 and another coercion &psi; from P2 to P3, then the composition of &phi; followed by &psi; must yield a (the) coercion from P1 to P3.</p>\n<h5>The identity is a coercion</h5>\n<p>There is a coercion from any parent into itself, namely the identity map. Together with the two preceding requirements, it follows: If there are conversions from P1 to P2 and from P2 to P1, then they are mutually inverse.</p>\n<h4>How to fulfill the above requirements, and preserve common sense</h4>\n<p>As an illustration, we discuss here the rationale behind the choice of coercions between multivariate polynomial rings in Sage. Let R be a ring.</p>\n<p>Obviously, there can be many ring homomorphisms between two polynomial rings over R. So, we have to fix a canonical choice. If the polynomial ring P2 has at least as many generators as P1, then one could map generator number n of P1 to generator number n of P2. Such \"place preserving\" conversions would certainly qualify as a coercion, since the composition of two place preserving conversions is a place preserving conversion.</p>\n<p>However, common sense would would more likely take the <em>names</em> of the generators into account: When mapping R[x,y] to R[y,x], one would rather map x to x and y to y, not the first generator x of R[x,y] to the first generator y of R[y,x].</p>\n<p>Sage tries to balance common sense and generality. If two polynomial rings have the same generators (but potentially in a different order), then there is a name preserving coercion:</p>"}︡
︠7df60400-69c6-469a-91bb-5473017fbe6e︠
P1.<x,y> = QQ[]; P2 = QQ['y,x,z']
︡02251140-d46c-40be-8733-fc9643354148︡︡
︠44721043-215f-4235-8628-1b85f76b7abe︠
f = P2.coerce_map_from(P1); f
︡df2da9b4-e132-4276-986c-9e68ef4b9f07︡{"stdout": "Call morphism:\n  From: Multivariate Polynomial Ring in x, y over Rational Field\n  To:   Multivariate Polynomial Ring in y, x, z over Rational Field"}︡
︠153c06ac-c8a3-44c5-96ef-3ef18600433d︠
f(x);f(y)
︡4c83294d-d9c7-463b-bffe-09ef16688bbd︡{"stdout": "x\ny"}︡
︠de278db6-a1ad-4726-8685-014d51b34e26i︠
%html
<p>However, if a name preserving map is not possible, then there is no coercion.</p>

︡be81fe82-80bf-4058-8f57-e6afcf2f081c︡{"html": "<p>However, if a name preserving map is not possible, then there is no coercion.</p>"}︡
︠4a68ca3f-661d-43f8-9abc-d1bbff384610︠
P3 = QQ['z,x']
P1.has_coerce_map_from(P3)
︡3a75e4fc-61a6-4728-ab10-dd2020c9f903︡{"stdout": "False"}︡
︠7a66fb09-f903-4360-a367-40436a2a1cd4i︠
%html
<p>Assume that we would use a place preserving map from P3 to P1 for coercion. Then, the composition with the name preserving coercion from P1 to P2 must be a coercion, too. But that composition is not name preserving and is thus different from the name preserving coercion from P3 to P2 that we already chose:</p>

︡f13e5877-91c5-4c4b-8287-6843e1f30260︡{"html": "<p>Assume that we would use a place preserving map from P3 to P1 for coercion. Then, the composition with the name preserving coercion from P1 to P2 must be a coercion, too. But that composition is not name preserving and is thus different from the name preserving coercion from P3 to P2 that we already chose:</p>"}︡
︠58aeb7a7-8726-416d-a315-46d6e761cd21︠
g = P2.coerce_map_from(P3)
P3.0, g(P3.0); P3.1,g(P3.1)
︡0e53b74b-f7b0-4f43-ba5b-89669293640f︡{"stdout": "(z, z)\n(x, x)"}︡
︠f36c179e-cfd9-4b46-b12a-4ffbaa5e018fi︠
%html
<p>By the requested uniqueness of coercions, a place preserving coercion from P3 to P1 is thus impossible.</p>
<p>One may use the place preserving map from P3 to P1 explicitly as a conversion, but it would never be used implicitly by Sage.</p>

︡fb13a8ef-cc8c-43ef-931b-e946c58705f7︡{"html": "<p>By the requested uniqueness of coercions, a place preserving coercion from P3 to P1 is thus impossible.</p>\n<p>One may use the place preserving map from P3 to P1 explicitly as a conversion, but it would never be used implicitly by Sage.</p>"}︡
︠886fe72a-456d-471a-ad0a-c1d6dd138b50︠
h = P1.convert_map_from(P3); h
︡22e6a6dd-d0a0-4303-b8aa-8792908bd01e︡{"stdout": "Call morphism:\n  From: Multivariate Polynomial Ring in z, x over Rational Field\n  To:   Multivariate Polynomial Ring in x, y over Rational Field"}︡
︠0a74f4df-7719-4b28-b337-3028413a17a1︠
P3.0;h(P3.0)
︡9962f04b-65be-4d5e-b86e-e955edd36df7︡{"stdout": "z\nx"}︡
︠c69b1e33-32be-4d01-a7b1-4d673134483e︠
P3.1;h(P3.1)
︡0bd0f41a-930f-49f2-9155-60bf1ba1842e︡{"stdout": "x\ny"}︡
︠3f1a7987-f9f0-4eed-83d7-a07b11cd0a9fi︠
%html
<h3>Implementing conversion</h3>
<p>We return to our example: Fraction fields. We have already seen that we could implement various conversions by simply providing the attribute "Element": We can convert from the base ring into the fraction field. However, we can not convert elements of a fraction field into elements of another fraction field, yet.</p>
<p>&nbsp;</p>

︡9f757bfb-353d-4e3b-a655-23eee14c8a2f︡{"html": "<h3>Implementing conversion</h3>\n<p>We return to our example: Fraction fields. We have already seen that we could implement various conversions by simply providing the attribute \"Element\": We can convert from the base ring into the fraction field. However, we can not convert elements of a fraction field into elements of another fraction field, yet.</p>\n<p>&nbsp;</p>"}︡
︠4815c757-9576-4817-9389-6dc80f95fd8f︠
P(2/3)
︡8a474a38-b407-4801-8599-a51b86552d9d︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_74.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"UCgyLzMp\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpwKdzSM/___code___.py\", line 3, in <module>\n    exec compile(u'P(_sage_const_2 /_sage_const_3 )' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"parent.pyx\", line 882, in sage.structure.parent.Parent.__call__ (sage/structure/parent.c:6462)\n  File \"coerce_maps.pyx\", line 82, in sage.structure.coerce_maps.DefaultConvertMap_unique._call_ (sage/structure/coerce_maps.c:3118)\n  File \"coerce_maps.pyx\", line 77, in sage.structure.coerce_maps.DefaultConvertMap_unique._call_ (sage/structure/coerce_maps.c:3021)\n  File \"/usr/local/sage2/local/lib/python2.6/site-packages/sage/categories/sets_cat.py\", line 284, in _element_constructor_from_element_class\n    return self.element_class(parent = self, *args, **keywords)\n  File \"\", line 9, in __init__\n    \nValueError: Numerator and denominator must be elements of Integer Ring"}︡
︠23178654-859d-4808-a1fa-32cd3ec17860i︠
%html
<p>Readers familiar with Python may assume that the conversion is  implemented by overriding the __call__ method of a parent. However, <strong>the default __call__ method should never be overridden.</strong></p>
<p>Instead, one should implement a method called _element_constructor_,  that accepts arguments to be converted, perhaps accepts optional  arguments, and returns an element of the parent. <strong>The returned element should be an instance of the parent's element class.</strong></p>
<p>We test whether the given argument is an element of a fraction field  by testing the category of its parent. If it is, then we can use  numerator and denominator. If it has no parent or the parent is no  fraction field, then we simply hope that our element class can deal with  the input.</p>
<p>Note that we return to the old name of the class (MyFrac, not  MyFracUnique), because otherwise an infinite recursion would occur in  the __classcall__ method. But uniqueness is still granted.</p>

︡d308ec9f-d71d-42d3-a2d3-6415b1b46785︡{"html": "<p>Readers familiar with Python may assume that the conversion is  implemented by overriding the __call__ method of a parent. However, <strong>the default __call__ method should never be overridden.</strong></p>\n<p>Instead, one should implement a method called _element_constructor_,  that accepts arguments to be converted, perhaps accepts optional  arguments, and returns an element of the parent. <strong>The returned element should be an instance of the parent's element class.</strong></p>\n<p>We test whether the given argument is an element of a fraction field  by testing the category of its parent. If it is, then we can use  numerator and denominator. If it has no parent or the parent is no  fraction field, then we simply hope that our element class can deal with  the input.</p>\n<p>Note that we return to the old name of the class (MyFrac, not  MyFracUnique), because otherwise an infinite recursion would occur in  the __classcall__ method. But uniqueness is still granted.</p>"}︡
︠d2948549-0ff6-44de-ac60-c282fcda093f︠
class MyFrac(MyFracUnique):
    def _element_constructor_(self, *args,**kwds):
        if len(args)!=1:
            return self.element_class(*args,parent=self,**kwds)
        x = args[0]
        try:
            P = x.parent()
        except AttributeError:
            return self.element_class(x,parent=self,**kwds)
        if P in QuotientFields() and P != self.base():
            return self.element_class(x.numerator(),x.denominator(),parent=self,**kwds)
        return self.element_class(x,parent=self,**kwds)
︡501db946-9671-488d-a183-22324415ad83︡︡
︠ddf348da-4acf-4a8c-8591-04b08ab8bcae︠
P = MyFrac(ZZ)
︡768219d7-8207-47d3-bce0-067452b6f55b︡︡
︠eaca0ec1-f250-4739-9f10-c769c8e2b43d︠
P is MyFrac(ZZ)
︡7061b09a-4bb5-4c33-bf90-26edd70b2ed5︡{"stdout": "True"}︡
︠40437b88-7e31-4dc8-8bc2-2d00730833bci︠
%html
<p>In addition to the conversion from the base ring and from pairs of base ring elements, we now also have a conversion from the rationals to our fraction field of ZZ:</p>

︡9993d11a-99b9-4c96-9acc-daad2bc80544︡{"html": "<p>In addition to the conversion from the base ring and from pairs of base ring elements, we now also have a conversion from the rationals to our fraction field of ZZ:</p>"}︡
︠e02b093a-e87b-4a08-acbc-c19ba8b8e406︠
P(2); P(2,3); P(3/4)
︡f18f8310-77d2-47ca-8e48-1e442702d0cd︡{"stdout": "(2):(1)\n(2):(3)\n(3):(4)"}︡
︠1ea758a5-99c2-4523-ab10-475fc38b0631i︠
%html
<h3>Establishing a coercion</h3>
<p>We have a conversion from the base ring and even from the rational field to P. But it is not known as a coercion:</p>

︡0d3da609-10f1-49cc-9080-f06bae528e92︡{"html": "<h3>Establishing a coercion</h3>\n<p>We have a conversion from the base ring and even from the rational field to P. But it is not known as a coercion:</p>"}︡
︠23c6c718-bc67-4ec5-aa56-689056dda2ae︠
P.has_coerce_map_from(ZZ), P.has_coerce_map_from(QQ)
︡7a903a93-095a-4c44-b73a-bbd1d1b2c8d1︡{"stdout": "(False, False)"}︡
︠23684667-a235-4afc-8d2f-f0e1b0c5f6eei︠
%html
<p>We could use P.register_coercion during initialisation (see documentation of that method) for declaring that there is a coercion from a ring to its fraction field. A more flexible way is to provide a method _coerce_map_from_ (with leading and trailing underscore). It accepts a parent as argument. If it returns an actual map, then this map is used for coercion. Otherwise, if it returns something that evaluates to "True", then conversion is used for coercion.</p>
<p>Note that we need a special case for the rational field, since QQ.base() is not the ring of integers.</p>

︡c78b1067-c7e4-4b4c-8c63-83b665c9a4c4︡{"html": "<p>We could use P.register_coercion during initialisation (see documentation of that method) for declaring that there is a coercion from a ring to its fraction field. A more flexible way is to provide a method _coerce_map_from_ (with leading and trailing underscore). It accepts a parent as argument. If it returns an actual map, then this map is used for coercion. Otherwise, if it returns something that evaluates to \"True\", then conversion is used for coercion.</p>\n<p>Note that we need a special case for the rational field, since QQ.base() is not the ring of integers.</p>"}︡
︠bf7260e3-6d80-403e-a2e1-2d4cabd34673︠
class MyFrac(MyFrac):
    def _coerce_map_from_(self, S):
        if self.base().has_coerce_map_from(S):
            return True
        if S in QuotientFields():
            if self.base().has_coerce_map_from(S.base()):
                return True
            if hasattr(S,'ring_of_integers') and self.base().has_coerce_map_from(S.ring_of_integers()):
                return True
︡8caaf67e-a499-4316-ac33-119838b7d8b1︡︡
︠23d0482c-64ce-4df8-9572-df92a4d965eci︠
%html
<p>By the method above, a parent coercing into the base ring will also coerce into the fraction field, and a fraction field coerces into another fraction field if there is a coercion of the corresponding base rings. Now, we have:</p>

︡a9042b81-d828-4d4a-9756-99b6bc84ebbd︡{"html": "<p>By the method above, a parent coercing into the base ring will also coerce into the fraction field, and a fraction field coerces into another fraction field if there is a coercion of the corresponding base rings. Now, we have:</p>"}︡
︠2aa5ceae-ee14-4c08-9941-278c39a9f877︠
P = MyFrac(QQ['x'])
︡9661c117-5833-4969-9b42-988b50772795︡︡
︠a00bb91a-3dad-46d3-bd11-f58a28d75b9a︠
P.has_coerce_map_from(ZZ['x']), P.has_coerce_map_from(Frac(ZZ['x'])), P.has_coerce_map_from(QQ)
︡f10edb3a-8e13-409d-bf08-163d68101596︡{"stdout": "(True, True, True)"}︡
︠b716a054-9286-464e-b876-53664ec15582i︠
%html
<p>We can no use coercion from ZZ['x'] and from QQ into P for arithmetic operations between the two rings:</p>

︡4d77ccde-6fc4-498f-99c7-6db061b300ca︡{"html": "<p>We can no use coercion from ZZ['x'] and from QQ into P for arithmetic operations between the two rings:</p>"}︡
︠696739ae-3ad7-4fd2-8f7b-3de6143253b9︠
3/4+P(2)+ZZ['x'].gen(), (P(2)+ZZ['x'].gen()).parent() is P
︡568aad38-bfb7-4939-954c-004132f03c1a︡{"stdout": "((4*x + 11):(4), True)"}︡
︠a7a67900-390c-4b39-b045-52b16417e787i︠
%html
<h3>Equality and element containment</h3>
<p>Recall that in above, the test "1 in P" returned False. Let us repeat the test now:</p>

︡3b396ad3-bbd1-4676-ac2b-141af90764ea︡{"html": "<h3>Equality and element containment</h3>\n<p>Recall that in above, the test \"1 in P\" returned False. Let us repeat the test now:</p>"}︡
︠8110af41-ee10-429a-baab-cec067317649︠
1 in P
︡fc84b4f6-17a1-44fe-9f04-f42b275c7ab5︡{"stdout": "True"}︡
︠d959b37d-f63b-4119-8aa6-c9c578834df5i︠
%html
<p>Why is that?</p>
<p>The default element containment test "x in P" is based on the interplay of three building blocks: conversion, coercion and equality test.</p>
<ol>
<li>Clearly, if the conversion P(x) raises an error, then x can not be seen as an element of P. On the other hand, a conversion P(x) can in general do very nasty things. So, the fact that P(x) works does not mean that x is in P.</li>
<li>If x.parent() is P, then the conversion P(x) will not change x (at least, that's the default). Hence, we will have x==P(x). But how can we have x==P(x) if x.parent() is not P?</li>
<li>Sage uses coercion not only for arithmetic operations, but also for comparison. Hence, if there is a coercion between the parent of x and P, then it will be applied. <em>If</em> there is a coercion from x.parent() to P then x==P(x) reduces to P(x)==P(x). Otherwise, x==P(x) will evaluate as false.</li>
</ol>
<p>That leads to the following default implementation of element containment testing:</p>
<p style="padding-left: 30px;">"x in P" holds if and only if "x==P(x)" does not raise an error and evaluates as true</p>
<p>If the user is not happy with that behaviour, the "magical" Python method __contains__ can be overridden.</p>
<h2>Coercion - the advanced parts</h2>
<p>So far, we are able to add integers and rational numbers to elements of our new implementation of the fraction field of ZZ.</p>

︡e4ce0540-31ff-4179-b989-927eb96544af︡{"html": "<p>Why is that?</p>\n<p>The default element containment test \"x in P\" is based on the interplay of three building blocks: conversion, coercion and equality test.</p>\n<ol>\n<li>Clearly, if the conversion P(x) raises an error, then x can not be seen as an element of P. On the other hand, a conversion P(x) can in general do very nasty things. So, the fact that P(x) works does not mean that x is in P.</li>\n<li>If x.parent() is P, then the conversion P(x) will not change x (at least, that's the default). Hence, we will have x==P(x). But how can we have x==P(x) if x.parent() is not P?</li>\n<li>Sage uses coercion not only for arithmetic operations, but also for comparison. Hence, if there is a coercion between the parent of x and P, then it will be applied. <em>If</em> there is a coercion from x.parent() to P then x==P(x) reduces to P(x)==P(x). Otherwise, x==P(x) will evaluate as false.</li>\n</ol>\n<p>That leads to the following default implementation of element containment testing:</p>\n<p style=\"padding-left: 30px;\">\"x in P\" holds if and only if \"x==P(x)\" does not raise an error and evaluates as true</p>\n<p>If the user is not happy with that behaviour, the \"magical\" Python method __contains__ can be overridden.</p>\n<h2>Coercion - the advanced parts</h2>\n<p>So far, we are able to add integers and rational numbers to elements of our new implementation of the fraction field of ZZ.</p>"}︡
︠88b1772c-5b07-4ea8-8c5e-55b5e5f653f0︠
P = MyFrac(ZZ)
︡f23fa17f-a8c2-4dc5-8cae-8a04235b6b52︡︡
︠68015fc2-a5d1-4fb7-aaeb-f8cde1c6d6e7︠
1/2+P(2,3)+1
︡42f1a7ec-80d9-46fa-838f-772f247a5100︡{"stdout": "(13):(6)"}︡
︠0a07a6e9-7478-4855-9c96-031eef4ee6bei︠
%html
<p>Surprisingly, we can even add a polynomial over the integers to an element of P, even though the the result lives in a new parent, namely in a polynomial ring over P:</p>

︡d7e90bba-eb52-4af5-b8b0-b750b1ea49df︡{"html": "<p>Surprisingly, we can even add a polynomial over the integers to an element of P, even though the the result lives in a new parent, namely in a polynomial ring over P:</p>"}︡
︠bb330d93-46a4-4354-bb91-eae3c2a0b003︠
P(1/2) + ZZ['x'].gen(), (P(1/2) + ZZ['x'].gen()).parent() is P['x']
︡d48f5ae1-1f90-4e77-a73c-4b8ce52080a4︡{"stdout": "((1):(1)*x + (1):(2), True)"}︡
︠a6ae8451-53a4-4359-a951-176ec7987e80i︠
%html
<p>That positive surprise may make us confident enough to try another, seemingly more easy example. There obviously is a coercion from the fraction field of ZZ to the fraction field of ZZ['x']. However, Sage does not know enough about our new implementation of fraction fields. Hence, it does not recognise the coercion:</p>

︡27f08d40-b717-42ed-a110-b131d27ac697︡{"html": "<p>That positive surprise may make us confident enough to try another, seemingly more easy example. There obviously is a coercion from the fraction field of ZZ to the fraction field of ZZ['x']. However, Sage does not know enough about our new implementation of fraction fields. Hence, it does not recognise the coercion:</p>"}︡
︠12b37278-0934-4a4f-9e7a-f703dd744fa0︠
Frac(ZZ['x']).has_coerce_map_from(P)
︡2369d5f8-521d-4925-917e-59fa12fe8fa8︡{"stdout": "False"}︡
︠26477a74-85d3-407b-abf6-8b272582447di︠
%html
<p>The aim of this section is to explain why and how a new ring was constructed when adding and element of P to an integral polynomial, and how we can establish a coercion from P to Frac(ZZ['x']).</p>
<p>Most parents can be constructed from simpler pieces. For example, the polynomial ring in variables x and y over the rational field can be obtained from the ring of integers by first applying the fraction field construction, then constructing a polynomial ring with generator x, and then constructing a polynomial ring with generator y (or: constructing a bivariate ring after the fraction field construction).</p>
<p>These constructions are functorial: The fraction field is a functor from the category of unique factorisation domains to the category of fields, and the construction of a polynomial ring with generator x is a functor from the category of rings into itself.</p>
<p>The advanced parts of Sage's coercion model are based on these functors, which are referred to as <em>construction functors</em>. If a parent is obtained from a simpler parent by applying a construction functor, then it should reveal it by a method `construction()`:</p>

︡8e7c75de-84b9-4038-8487-f6fb07249a2f︡{"html": "<p>The aim of this section is to explain why and how a new ring was constructed when adding and element of P to an integral polynomial, and how we can establish a coercion from P to Frac(ZZ['x']).</p>\n<p>Most parents can be constructed from simpler pieces. For example, the polynomial ring in variables x and y over the rational field can be obtained from the ring of integers by first applying the fraction field construction, then constructing a polynomial ring with generator x, and then constructing a polynomial ring with generator y (or: constructing a bivariate ring after the fraction field construction).</p>\n<p>These constructions are functorial: The fraction field is a functor from the category of unique factorisation domains to the category of fields, and the construction of a polynomial ring with generator x is a functor from the category of rings into itself.</p>\n<p>The advanced parts of Sage's coercion model are based on these functors, which are referred to as <em>construction functors</em>. If a parent is obtained from a simpler parent by applying a construction functor, then it should reveal it by a method `construction()`:</p>"}︡
︠7f7663b6-6925-4dea-b23d-4a1839a9aec6︠
F2,R = QQ[x].construction()
F2,R
︡fbf18d8b-f564-4668-a352-8db5e8bb50c4︡{"stdout": "(Poly[x], Rational Field)"}︡
︠03a734be-0e61-4368-9853-9d70f2713cf8︠
F2(R) is QQ[x]
︡788bf8e6-48b2-4d48-a4fb-39c2300d3d4d︡{"stdout": "True"}︡
︠5a350a66-5448-4095-bfba-8d196ae9c80b︠
F1,R = QQ.construction()
F1,R
︡0f6a45c4-e802-41f4-a016-68ed6a547a65︡{"stdout": "(FractionField, Integer Ring)"}︡
︠679b4177-6a92-421c-82f7-07b8fe1f9337︠
F2*F1
︡d3e5720e-ff31-4214-b59f-a367b15ac557︡{"stdout": "Poly[x](FractionField(...))"}︡
︠3aa53658-609c-4a85-a873-91c98d01c9bd︠
(F2*F1)(R) is QQ[x]
︡b10a5e8a-7582-4214-92de-f8c6a873cd30︡{"stdout": "True"}︡
︠cb664487-c4c6-4b08-97a2-72ad65adfd35i︠
%html
<p>Note that construction functors do not necessarily commute: The polynomial ring over the fraction field of the integers is not the same as the fraction field of the polynomial ring over the integers.</p>

︡42988e0e-5473-488a-9b58-8483b4d18ebc︡{"html": "<p>Note that construction functors do not necessarily commute: The polynomial ring over the fraction field of the integers is not the same as the fraction field of the polynomial ring over the integers.</p>"}︡
︠1e43104d-e643-4e11-ac56-113b7aa10e94︠
(F1*F2)(R)
︡76cb61c0-8056-4b04-a2cd-129f5f861ec2︡{"stdout": "Fraction Field of Univariate Polynomial Ring in x over Integer Ring"}︡
︠ef50f32d-231e-4ffe-8bf2-4434a2e98bd4i︠
%html
<p>Now, let P1 and P2 be two parents that are obtained from the same parent R by applying two different construction functors F1, F2.</p>
<p>It is often possible to combine F1 and F2 to a new construction functor F3, such that F3(R) is a new parent P3 which both P1 and P2 coerce into. In analogy to a notion of category theory, P3 is called the <em>pushout</em> of P1 and P2; and similarly F3 is called the pushout of F1 and F2.</p>

︡895ce65c-6271-4a2a-b400-578a164d95c8︡{"html": "<p>Now, let P1 and P2 be two parents that are obtained from the same parent R by applying two different construction functors F1, F2.</p>\n<p>It is often possible to combine F1 and F2 to a new construction functor F3, such that F3(R) is a new parent P3 which both P1 and P2 coerce into. In analogy to a notion of category theory, P3 is called the <em>pushout</em> of P1 and P2; and similarly F3 is called the pushout of F1 and F2.</p>"}︡
︠922912a2-4fc0-4c18-af92-032cc59ad746︠
from sage.categories.pushout import pushout
pushout(F1(R),F2(R))
︡d556c8ff-24e9-4cd6-a0cb-f8ccbf8ebe62︡{"stdout": "Univariate Polynomial Ring in x over Rational Field"}︡
︠f8f95909-9f4c-4604-a708-7851b694547bi︠
%html
<p>Obviously, both F1*F2 and F2*F1 are natural candidates for the pushout of F1 and F2. But the pushout must not depend on the order in which F1 and F2 are provided. Hence, there must be a consistent choice.</p>
<p>Any construction functor has an attribute `rank`, and the convention is that if F1.rank is smaller than F2.rank, then the pushout is F2*F1 (hence, F1 is applied first, since the multiplication is in functorial notation). We have</p>

︡9c09c11e-5e24-4993-875a-c15892972576︡{"html": "<p>Obviously, both F1*F2 and F2*F1 are natural candidates for the pushout of F1 and F2. But the pushout must not depend on the order in which F1 and F2 are provided. Hence, there must be a consistent choice.</p>\n<p>Any construction functor has an attribute `rank`, and the convention is that if F1.rank is smaller than F2.rank, then the pushout is F2*F1 (hence, F1 is applied first, since the multiplication is in functorial notation). We have</p>"}︡
︠4d93d4ef-c5b3-47c7-934f-eeb674506d50︠
F1.rank, F2.rank
︡d6a50b95-7237-4ff2-970a-e521988532fa︡{"stdout": "(5, 9)"}︡
︠c93c977e-5d34-4500-8db8-2a6916de798ci︠
%html
<p>and thus the pushout is</p>

︡0954571b-37b0-445d-8d75-8aa6e6256643︡{"html": "<p>and thus the pushout is</p>"}︡
︠ff885aca-5afa-4134-852b-c35a0b2f707b︠
F1.pushout(F2), F2.pushout(F1)
︡bed4571c-9e9d-48db-8b88-b26d46580d9e︡{"stdout": "(Poly[x](FractionField(...)), Poly[x](FractionField(...)))"}︡
︠ab359442-1140-4ad9-99b8-d8e8349e009ci︠
%html
<p>regardless of the order in which F1 and F2 are provided. That is why the sum of a rational number and a polynomial over the integers is a polynomial over the rationals, and not an element of the fraction field of the polynomials over the integers.</p>
<p>However, only "elementary" construction functors have a rank:</p>

︡6817c436-7922-4158-9fcd-048fd7ccebb0︡{"html": "<p>regardless of the order in which F1 and F2 are provided. That is why the sum of a rational number and a polynomial over the integers is a polynomial over the rationals, and not an element of the fraction field of the polynomials over the integers.</p>\n<p>However, only \"elementary\" construction functors have a rank:</p>"}︡
︠0bf7e837-df57-4aa9-bd48-d080650b77d6︠
(F1*F2).rank
︡2f48e342-b083-48d3-a04c-b54e0fdaa9cd︡{"stderr": "Traceback (most recent call last):\n  File \"<stdin>\", line 1, in <module>\n  File \"_sage_input_98.py\", line 10, in <module>\n    exec compile(u'open(\"___code___.py\",\"w\").write(\"# -*- coding: utf-8 -*-\\\\n\" + _support_.preparse_worksheet_cell(base64.b64decode(\"KEYxKkYyKS5yYW5r\"),globals())+\"\\\\n\"); execfile(os.path.abspath(\"___code___.py\"))' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \n  File \"/tmp/tmpIFRBmi/___code___.py\", line 2, in <module>\n    exec compile(u'(F1*F2).rank' + '\\n', '', 'single')\n  File \"\", line 1, in <module>\n    \nAttributeError: 'CompositConstructionFunctor' object has no attribute 'rank'"}︡
︠ea39bda0-e824-4a9b-aaac-34f21ee22049i︠
%html
<p>So, what do we do if P1 and P2 are obtained from R by two <em>chains </em>of construction functors?</p>
<p>The two functor chains will be shuffled: If the rank of the next unused functor F1 of the first chain is smaller (or greater) than the rank of the next unused functor F2 of the second chain, then F1 (or F2) is applied, and the next functor in the first (or second) chain is considered. If the ranks of F1 and F2 coincide, then we need another idea that is explained below.</p>
<p>As an illustration, we first get us some functors and then see how chains of functors are shuffled.</p>

︡d613ff43-3fe3-4064-bbd2-1b788c1587bd︡{"html": "<p>So, what do we do if P1 and P2 are obtained from R by two <em>chains </em>of construction functors?</p>\n<p>The two functor chains will be shuffled: If the rank of the next unused functor F1 of the first chain is smaller (or greater) than the rank of the next unused functor F2 of the second chain, then F1 (or F2) is applied, and the next functor in the first (or second) chain is considered. If the ranks of F1 and F2 coincide, then we need another idea that is explained below.</p>\n<p>As an illustration, we first get us some functors and then see how chains of functors are shuffled.</p>"}︡
︠6f793009-cf50-4af5-bea8-63a3b1002958︠
F3, R = CC.construction(); F3
︡22b1cfba-1875-4788-911a-0466b9ee61f7︡{"stdout": "AlgebraicClosureFunctor"}︡
︠655d54be-604a-44c6-b48f-4f1cc6b4443d︠
F4, R = RR.construction(); F4
︡f0133bfb-d9d7-40dd-a5a9-0cfa3e477618︡{"stdout": "CompletionFunctor"}︡
︠56a9b62c-0943-4547-b922-adc5b5ff760d︠
F5, R = (MatrixSpace(ZZ,3)).construction(); F5
︡c95d6ef2-7d15-44f3-866f-b5ed4708b844︡{"stdout": "MatrixFunctor"}︡
︠7aac45c0-8649-425b-be9c-7600c24591ee︠
F1.rank, F2.rank, F3.rank, F4.rank, F5.rank
︡0faa52ea-0e51-4226-87aa-bf8d6f8fd3bf︡{"stdout": "(5, 9, 3, 4, 10)"}︡
︠2520bfb4-a24d-4695-bc61-ceb8cc98eee5i︠
%html
<p>When we apply F1, F3, F2 and F1 to the ring of integers, we obtain</p>

︡8980ab6e-114a-47ae-b83f-0c8decb2dd73︡{"html": "<p>When we apply F1, F3, F2 and F1 to the ring of integers, we obtain</p>"}︡
︠0a4fc8dd-ced3-4c6d-a6ab-61505bbb59fc︠
(F1*F2*F3*F1)(ZZ)
︡348347e8-3874-465e-9ba9-5f7798bca91d︡{"stdout": "Fraction Field of Univariate Polynomial Ring in x over Algebraic Field"}︡
︠c83ec4fb-fa6a-4591-b0ea-255c59246697i︠
%html
<p>When we apply F4, F5 and F1 to the ring of integers, we obtain</p>

︡a9935a5e-6101-4d7d-9873-d253dd0565da︡{"html": "<p>When we apply F4, F5 and F1 to the ring of integers, we obtain</p>"}︡
︠f06a2535-ee0f-413a-88c9-fbf901616b8d︠
(F2*F5*F4)(ZZ)
︡8333f882-2e8f-4516-ac32-c71295ffaf22︡{"stdout": "Univariate Polynomial Ring in x over Full MatrixSpace of 3 by 3 dense matrices over Real Field with 53 bits of precision"}︡
︠719a75d2-fb41-4de8-aa35-87275f2c785di︠
%html
<p>When we shuffle F1,F3,F2,F1 with F4,F5,F2 according to the ranks, we start with F4 (second chain), since F4.rank&lt;F1.rank. Then we proceed with with the complete first chain, since F1.rank, F3.rank and F2.rank are all smaller than F5.rank. We finish with the remaining functors of the second chain, F5 and F2.</p>
<p>So, we expect that the pushout of the above parents is obtained by applying F4, F1, F3, F2, F1, F5 and F2 to the ring of integers. That is indeed the case:</p>

︡4dba14c2-9952-4adf-831e-ce82efb6c742︡{"html": "<p>When we shuffle F1,F3,F2,F1 with F4,F5,F2 according to the ranks, we start with F4 (second chain), since F4.rank&lt;F1.rank. Then we proceed with with the complete first chain, since F1.rank, F3.rank and F2.rank are all smaller than F5.rank. We finish with the remaining functors of the second chain, F5 and F2.</p>\n<p>So, we expect that the pushout of the above parents is obtained by applying F4, F1, F3, F2, F1, F5 and F2 to the ring of integers. That is indeed the case:</p>"}︡
︠e66416e3-63bb-472f-8837-fbc4261b16d4︠
(F2*F5*F1*F2*F3*F1*F4)(ZZ)
︡d7f437de-5fa3-42b3-848e-ff9a692e6f67︡{"stdout": "Univariate Polynomial Ring in x over Full MatrixSpace of 3 by 3 dense matrices over Fraction Field of Univariate Polynomial Ring in x over Complex Field with 53 bits of precision"}︡
︠6d4a0abf-88ca-45c4-88f6-b95c6e52fa7b︠
pushout((F1*F2*F3*F1)(ZZ), (F2*F5*F4)(ZZ))
︡80ac5196-7478-4c90-98e3-779dd03301cf︡{"stdout": "Univariate Polynomial Ring in x over Full MatrixSpace of 3 by 3 dense matrices over Fraction Field of Univariate Polynomial Ring in x over Complex Field with 53 bits of precision"}︡
︠90e292af-5cf0-409c-83e4-c88deb0f7a90i︠
%html
<p>There remains the case that the next unused functor F1 of the first chain has the same rank as the next unused functor F2 of the second chain. Sage's pushout constructions offers two ways to proceed:</p>
<ol>
<li>Construction functors have a method `commutes()`. If F1.commutes(F2) or F2.commutes(F1) return True, then it is guaranteed that it does not matter in which order F1 and F2 are applied to any given parent R. So, we apply both F1 and F2, in any order.</li>
<li>Construction functors have a method `merge()` that either returns None or returns a construction functor. By default, F1.merge(F2) is F1 if F1==F2, and is None otherwise. If F1.merge(F2) returns a functor F3, then F3 is applied, and both F1 and F2 are discarded.</li>
</ol>
<p>We remark that the `commutes` method exists, but it seems that so far nobody has implemented two functors of the same rank that commute, although some functors of different ranks do commute:</p>

︡0c0ae0da-4f94-4c3a-8aa1-7ffa20f187b1︡{"html": "<p>There remains the case that the next unused functor F1 of the first chain has the same rank as the next unused functor F2 of the second chain. Sage's pushout constructions offers two ways to proceed:</p>\n<ol>\n<li>Construction functors have a method `commutes()`. If F1.commutes(F2) or F2.commutes(F1) return True, then it is guaranteed that it does not matter in which order F1 and F2 are applied to any given parent R. So, we apply both F1 and F2, in any order.</li>\n<li>Construction functors have a method `merge()` that either returns None or returns a construction functor. By default, F1.merge(F2) is F1 if F1==F2, and is None otherwise. If F1.merge(F2) returns a functor F3, then F3 is applied, and both F1 and F2 are discarded.</li>\n</ol>\n<p>We remark that the `commutes` method exists, but it seems that so far nobody has implemented two functors of the same rank that commute, although some functors of different ranks do commute:</p>"}︡
︠3774d2f3-7185-44f1-92f8-a26bb99c8fa1︠
F4.commutes(F1), F4.rank==F1.rank   # The answer is (True, False), if #8807 is merged
︡82e80735-bb81-471a-8fee-69b9e89672a7︡{"stdout": "(False, False)"}︡
︠9b3e7bf2-a799-46bc-94c7-7c027597bf01i︠
%html
<p>The `merge` method, however, plays an important role. Typically it is used to <strong>provide coercion between different implementations of the same algebraic structure:</strong> For example, F1(R) may be the dense implementation of a polynomial ring and and F2(R) a sparse implementation. Then, F1.merge(F2) should be a functor F3, such that F3(R) is the same algebraic structure as F1(R) and F2(R), in a default implementation.</p>
<p>Here, we have two implementations of fraction fields, namely the one that already existed in Sage, and the new implementation that is the main example of our tutorial. We are now creating a construction functor `MyFracFunctor`, and merge it with the usual fraction field functor in such a way that the new functor becomes the default.</p>
<p>Since functors will only be merged if their ranks coincide, our new functor must have the same rank as the usual fraction field construction functor, namely 5. It is a functor from the category of unique factorization domains to the category of fields.</p>
<p>Note that since sage-4.7, the application of a functor to an object of a category should be implemented using the method `_apply_functor`. <strong>Do not override the default __call__ method!<br /></strong></p>

︡1ce2544a-eabf-4235-9946-62bd1c13d7c8︡{"html": "<p>The `merge` method, however, plays an important role. Typically it is used to <strong>provide coercion between different implementations of the same algebraic structure:</strong> For example, F1(R) may be the dense implementation of a polynomial ring and and F2(R) a sparse implementation. Then, F1.merge(F2) should be a functor F3, such that F3(R) is the same algebraic structure as F1(R) and F2(R), in a default implementation.</p>\n<p>Here, we have two implementations of fraction fields, namely the one that already existed in Sage, and the new implementation that is the main example of our tutorial. We are now creating a construction functor `MyFracFunctor`, and merge it with the usual fraction field functor in such a way that the new functor becomes the default.</p>\n<p>Since functors will only be merged if their ranks coincide, our new functor must have the same rank as the usual fraction field construction functor, namely 5. It is a functor from the category of unique factorization domains to the category of fields.</p>\n<p>Note that since sage-4.7, the application of a functor to an object of a category should be implemented using the method `_apply_functor`. <strong>Do not override the default __call__ method!<br /></strong></p>"}︡
︠8f5b7a55-84e8-411c-adeb-c06c69ad060f︠
from sage.categories.pushout import ConstructionFunctor
class MyFracFunctor(ConstructionFunctor):
    rank = 5
    def __init__(self):
        if is_9138_merged:
            ConstructionFunctor.__init__(self, UniqueFactorizationDomains(), Fields())
        else:
            ConstructionFunctor.__init__(self, Rings(), Fields())
    def __repr__(self):
        return "BetterFractionField"
    def _apply_functor(self, R):
        return MyFrac(R)
    def merge(self, other):
        if isinstance(other, (type(self), sage.categories.pushout.FractionField)):
            return self
︡c7f570cd-45b0-4629-be15-67564444b4cb︡︡
︠b014f075-c07f-4fc7-aaf6-f0d906eb0bdci︠
%html
<p>However, at the time of writing this tutorial, the main sage notebook server was still using sage-4.6. In that version, MyFracFunctor would return none, which is a bug fixed in trac ticket #8800. Hence, we use a little work-around.</p>

︡b732e656-3680-4270-b83d-e82b6b5d064c︡{"html": "<p>However, at the time of writing this tutorial, the main sage notebook server was still using sage-4.6. In that version, MyFracFunctor would return none, which is a bug fixed in trac ticket #8800. Hence, we use a little work-around.</p>"}︡
︠4d288e8b-caaa-4a0f-be6b-d5255434e538︠
if MyFracFunctor()(ZZ) is None:
    class MyFracFunctor(MyFracFunctor):
        def __call__(self, R):
            return self._apply_functor(R)
︡ff41ac37-82c6-4e96-b255-6d2ac0761016︡︡
︠c1e7c1c6-4858-4090-a750-cb4c40b8813fi︠
%html
<p>We verify that our functor can really be used to construct our implementation of fraction fields, and that it can be merged with either itself or the usual fraction field constructor:</p>

︡672b03b4-4af9-47ef-aea2-0b8938f361dc︡{"html": "<p>We verify that our functor can really be used to construct our implementation of fraction fields, and that it can be merged with either itself or the usual fraction field constructor:</p>"}︡
︠50f16338-851c-4c8d-ab83-607bfec00655︠
MyFracFunctor()(ZZ)
︡68113aa9-371a-49c3-b121-c0099f90eac1︡{"stdout": "NewFrac(Integer Ring)"}︡
︠c147ccd6-b965-465d-99c1-d85832654cd9︠
MyFracFunctor().merge(MyFracFunctor())
︡fb2e1969-615a-42b1-afb5-c458eeb38af4︡{"stdout": "BetterFractionField"}︡
︠2f05af5a-4ed1-4f5f-8094-567cac7370d0︠
MyFracFunctor().merge(F1)
︡c253c590-7efa-46a7-b6a5-6fc2db91853f︡{"stdout": "BetterFractionField"}︡
︠94865a7a-fed1-4105-b3d8-4848dc6fa93bi︠
%html
<p>There remains to let our new fraction fields know about the new construction functor:</p>

︡e13c0690-3841-4eba-b418-6130b120fa88︡{"html": "<p>There remains to let our new fraction fields know about the new construction functor:</p>"}︡
︠b73119c2-a54e-4510-9ae0-6c75aa5b3921︠
class MyFrac(MyFrac):
    def construction(self):
        return MyFracFunctor(), self.base()
︡0af7856f-d8ef-412b-9277-b87a62330804︡︡
︠e2d39c69-6f8c-4a8f-9e08-f2ac548bc9c5︠
Frac(ZZ['x']).construction()
︡1569df70-468d-428d-bf27-9cbc16ea5931︡{"stdout": "(FractionField, Univariate Polynomial Ring in x over Integer Ring)"}︡
︠fd5d9c63-f2dd-4064-bb42-e1742bd29c26i︠
%html
<p>Due to merging, we have:</p>

︡fabae17d-bd1d-4cc8-9d17-537089fd4fa2︡{"html": "<p>Due to merging, we have:</p>"}︡
︠887a92c4-6e1d-44bb-b1d2-22dee74526e2︠
pushout(MyFrac(ZZ['x']), Frac(QQ['x']))
︡7dd6a6a3-f2dd-4adf-a270-a49be89dd797︡{"stdout": "NewFrac(Univariate Polynomial Ring in x over Rational Field)"}︡
︠7c5dce6b-c2d0-4f49-b0b9-71acfc296eebi︠
%html
<p>Note that merging of functors really occurs here: There is no coercion from QQ['x'] to ZZ['x'].</p>
<h2>The Test Suites</h2>
<p>We have already mentioned that the category framework does not only provide functionality but&nbsp; also a test framework. The aim of this last section is to show how one can add more tests.</p>
<p>Parents or elements that belong to a certain category must provide certain methods, in order to smoothly blend with the methods that already exist in Sage. The methods that ought to be provided are called "abstract methods", and they can be required or optional. Let us see what methods are needed for quotient fields and their elements:</p>

︡5c8012d8-9c87-4e1e-a124-021d23027ca0︡{"html": "<p>Note that merging of functors really occurs here: There is no coercion from QQ['x'] to ZZ['x'].</p>\n<h2>The Test Suites</h2>\n<p>We have already mentioned that the category framework does not only provide functionality but&nbsp; also a test framework. The aim of this last section is to show how one can add more tests.</p>\n<p>Parents or elements that belong to a certain category must provide certain methods, in order to smoothly blend with the methods that already exist in Sage. The methods that ought to be provided are called \"abstract methods\", and they can be required or optional. Let us see what methods are needed for quotient fields and their elements:</p>"}︡
︠c2ebc597-be40-4889-8e1e-1b96df992fa0︠
from sage.misc.abstract_method import abstract_methods_of_class
︡4c6df3f5-913a-496a-a9ad-e0391967ed58︡︡
︠f0dfe983-f5d9-4b10-a143-ea1eded8118a︠
abstract_methods_of_class(QuotientFields().parent_class)
︡f2d71998-1f8b-4136-a79b-0761e7956342︡{"stdout": "{'required': ['__contains__'], 'optional': []}"}︡
︠9ed8778d-a9f4-4892-8d7a-31ff8ad3a039i︠
%html
<p>Hence, the only required method (that is actually required for all parents that belong to the category of sets) is an element containment test. That's fine, because the base class `sage.structure.parent.Parent` provides a default containment test.</p>
<p>The elements have to fulfill more:</p>

︡4c585950-f44f-431f-9f44-53b4cd20784a︡{"html": "<p>Hence, the only required method (that is actually required for all parents that belong to the category of sets) is an element containment test. That's fine, because the base class `sage.structure.parent.Parent` provides a default containment test.</p>\n<p>The elements have to fulfill more:</p>"}︡
︠b10e80a1-21fd-46a9-bd5e-be6289989277︠
abstract_methods_of_class(QuotientFields().element_class)
︡c40598f6-b09e-4f06-bb32-e574ce32c65b︡{"stdout": "{'required': ['denominator', 'numerator'], 'optional': ['_add_', '_mul_']}"}︡
︠9c84a825-2e14-4ab5-a6e2-b1fbe3f90055i︠
%html
<p>Hence, the elements may have Sage's single underscore arithmetic methods (actually any ring element <em>should</em> have them).</p>
<p>Then, there are test methods defined for the parent or element class of a category. Their names start with "_test_". They test, for example, whether a parent P actually is an instance of the parent class of the category of P. Or, if the definition of category defines certain properties such as commutativity, then that property is tested. Or the existence of required abstract methods is tested.</p>
<p>Here are the tests that form the test suite of quotient fields:</p>

︡fccba098-f6cc-42e2-ad91-b4abcfb8610e︡{"html": "<p>Hence, the elements may have Sage's single underscore arithmetic methods (actually any ring element <em>should</em> have them).</p>\n<p>Then, there are test methods defined for the parent or element class of a category. Their names start with \"_test_\". They test, for example, whether a parent P actually is an instance of the parent class of the category of P. Or, if the definition of category defines certain properties such as commutativity, then that property is tested. Or the existence of required abstract methods is tested.</p>\n<p>Here are the tests that form the test suite of quotient fields:</p>"}︡
︠240ef5cb-1cd1-422a-a11b-e22da482ed03︠
[t for t in dir(QuotientFields().parent_class) if t.startswith('_test_')]
︡354945f3-b71c-4575-bb82-b8a50c6ba63d︡{"stdout": "['_test_additive_associativity', '_test_an_element', '_test_associativity', '_test_distributivity', '_test_elements', '_test_elements_eq', '_test_one', '_test_prod', '_test_some_elements', '_test_zero']"}︡
︠eb177057-fad5-4243-8e55-a5654b941919i︠
%html
<p>Since we have implemented all abstract methods, we are confident that the test suite runs without complaining. In fact, it does!</p>

︡01de4e89-a830-4721-9a53-87675ec67b8e︡{"html": "<p>Since we have implemented all abstract methods, we are confident that the test suite runs without complaining. In fact, it does!</p>"}︡
︠3bb5dc62-7658-4280-8bd7-55caefecb6f1︠
P = MyFrac(ZZ['x'])
︡451b2814-083e-4aa4-8ec2-711df14a8357︡︡
︠cdc4a3bf-a89c-413d-bb92-ef887c89e434︠
TestSuite(P).run()
︡78edff3a-42db-4689-83c0-d9209c70bf2e︡︡
︠620e9e45-ebac-428d-b8f0-b793235b5736i︠
%html
<p>Let us see what tests are actually performed:</p>

︡e8688bb2-8792-4acb-8b87-a43bc34f9d54︡{"html": "<p>Let us see what tests are actually performed:</p>"}︡
︠b6ced3a7-8427-4454-bc5c-ba120e36e9ea︠
TestSuite(P).run(verbose=True)
︡be9db4a0-67ae-4954-a4d2-e4e1627df7e3︡{"stdout": "running ._test_additive_associativity() . . . pass\nrunning ._test_an_element() . . . pass\nrunning ._test_associativity() . . . pass\nrunning ._test_category() . . . pass\nrunning ._test_distributivity() . . . pass\nrunning ._test_elements() . . .\n  Running the test suite of self.an_element()\n  running ._test_category() . . . pass\n  running ._test_eq() . . . pass\n  running ._test_not_implemented_methods() . . . pass\n  running ._test_pickling() . . . pass\n  pass\nrunning ._test_elements_eq() . . . pass\nrunning ._test_eq() . . . pass\nrunning ._test_not_implemented_methods() . . . pass\nrunning ._test_one() . . . pass\nrunning ._test_pickling() . . . pass\nrunning ._test_prod() . . . pass\nrunning ._test_some_elements() . . . pass\nrunning ._test_zero() . . . pass"}︡
︠daa59fff-cf8a-45f3-82dc-45974e683c65i︠
%html
<p>As one can see, tests are also performed on elements. There are methods that return one element or a list of some elements, relying on "typical" elements that can be found in most algebraic structures".</p>

︡fafef11f-b379-4fab-98a2-17d48a9d6fcf︡{"html": "<p>As one can see, tests are also performed on elements. There are methods that return one element or a list of some elements, relying on \"typical\" elements that can be found in most algebraic structures\".</p>"}︡
︠792e4771-1a54-4d81-b088-2d3239591fdb︠
P.an_element(); P.some_elements()
︡c6d55d0e-6f68-449a-a5d0-aafe8ce62d37︡{"stdout": "(2):(1)\n[(2):(1)]"}︡
︠8a2b1a88-2676-424c-8ef7-8b4fd4896662i︠
%html
<p>Unfortunately, the list of elements that is returned by the default  method is of length one, and that single element could also be a bit  more interesting.</p>
<p>The method an_element relies on a method _an_element_, so, we implement that. We also override the some_elements method.</p>

︡846d666a-953c-48bc-bef2-6ff5388251b0︡{"html": "<p>Unfortunately, the list of elements that is returned by the default  method is of length one, and that single element could also be a bit  more interesting.</p>\n<p>The method an_element relies on a method _an_element_, so, we implement that. We also override the some_elements method.</p>"}︡
︠a8669a08-ecb6-437d-bebf-d917641704c0︠
class MyFrac(MyFrac):
    def _an_element_(self):
        a = self.base().an_element()
        b = self.base_ring().an_element()
        if (a+b)!=0:
            return self(a)**2/(self(a+b)**3)
        if b != 0:
            return self(a)/self(b)**2
        return self(a)**2*self(b)**3
    def some_elements(self):
        return [self.an_element(),self(self.base().an_element()),self(self.base_ring().an_element())]
︡4dac6ad5-fb3a-47b9-8222-085b28c203ae︡︡
︠001baaef-e3a5-4030-8857-dd1a7d5697d0︠
P = MyFrac(ZZ['x'])
P.an_element(); P.some_elements()
︡a45de6e0-4ab2-4c61-9f29-70f9593cedbf︡{"stdout": "(x^2):(x^3 + 3*x^2 + 3*x + 1)\n[(x^2):(x^3 + 3*x^2 + 3*x + 1), (x):(1), (1):(1)]"}︡
︠7e2a0c47-1b9b-4e44-917a-f9f373aa24a4i︠
%html
<p>Now, as we have more&nbsp; interesting elements, we may also add a test for the "factor" method. Recall that the method was inherited from the category, but it appears that it is not tested. The tests that form the test suites should be implemented as a parent or element method of a category (and in addition, each method should have a documentation with tests, but that's a different topic).</p>
<p>Normally, a test for a method defined by a category should be provided by the same category. Hence, since `factor()` is defined in the category of quotient fields, a test should be added there. But we won't change source code here and will instead create a sub-category.</p>
<p>Apparently, the product of the factors returned be `e.factor()` should be equal to `e`. For forming the product, we use the `prod()` method, that, no surprise, is inherited from another category.</p>

︡dc3f2fef-02f4-4d63-b74d-55f993b02a98︡{"html": "<p>Now, as we have more&nbsp; interesting elements, we may also add a test for the \"factor\" method. Recall that the method was inherited from the category, but it appears that it is not tested. The tests that form the test suites should be implemented as a parent or element method of a category (and in addition, each method should have a documentation with tests, but that's a different topic).</p>\n<p>Normally, a test for a method defined by a category should be provided by the same category. Hence, since `factor()` is defined in the category of quotient fields, a test should be added there. But we won't change source code here and will instead create a sub-category.</p>\n<p>Apparently, the product of the factors returned be `e.factor()` should be equal to `e`. For forming the product, we use the `prod()` method, that, no surprise, is inherited from another category.</p>"}︡
︠6925a71d-50c5-4e8b-9952-e327d4693bbe︠
P.prod.__module__
︡3a1e267d-8458-4a52-977b-c3ebc3930ba1︡{"stdout": "'sage.categories.monoids'"}︡
︠0af3c3a8-5a99-41d7-ac3a-6b18d39a5241i︠
%html
<p>When we want to create a sub-category, we need to provide a method `super_categories`, that returns a list of all immediate super categories (here: category of quotient fields); it should be a cached method, for efficiency. The parent and element methods of a category are provided as methods of a class `ParentMethods` and `Element Methods`, as follows:</p>

︡e3030174-10b5-441e-bb9c-058200393671︡{"html": "<p>When we want to create a sub-category, we need to provide a method `super_categories`, that returns a list of all immediate super categories (here: category of quotient fields); it should be a cached method, for efficiency. The parent and element methods of a category are provided as methods of a class `ParentMethods` and `Element Methods`, as follows:</p>"}︡
︠d166468d-1ce0-473d-99da-23580b99cac1︠
from sage.categories.category import Category
class QuotientFieldsWithTest(Category):
    @cached_method
    def super_categories(self):
        return [QuotientFields()]
    class ParentMethods:
        pass
    class ElementMethods:
        def _test_factorisation(self, **options):
            P = self.parent()
            assert self == P.prod([P(b)**e for b,e in self.factor()])
︡f3391b7d-c3d5-476d-ac9f-2c19b8ee3315︡︡
︠81df99d6-b633-4a5c-812a-095798b9a4b5i︠
%html
<p>We provide an instance of our quotient field implementation with that new category.</p>

︡2c2fef8b-de9c-4cb5-a9c1-21614fb4ecb0︡{"html": "<p>We provide an instance of our quotient field implementation with that new category.</p>"}︡
︠0716f26e-e6a1-4f33-a670-bb29ed61710f︠
P = MyFrac(ZZ['x'], category=QuotientFieldsWithTest())
P.category()
︡0971ae50-d390-453c-82cc-9be25ea0d3c4︡{"stdout": "Category of quotient fields with test"}︡
︠39ebca77-2439-4194-9ab8-5b865d45d26di︠
%html
<p>The new test is inherited from the category. Since an_element() is returning a complicated element, _test_factorisation is a serious test.</p>

︡cdcc674e-3c4a-4db6-baeb-76a5aa295f4e︡{"html": "<p>The new test is inherited from the category. Since an_element() is returning a complicated element, _test_factorisation is a serious test.</p>"}︡
︠2d7430a4-dcb8-4777-ae4d-dcdb6d2b2427︠
P.an_element()._test_factorisation
︡334409d5-ca69-4849-b021-c59c62242293︡{"stdout": "<bound method MyFrac_with_category.element_class._test_factorisation of (x^2):(x^3 + 3*x^2 + 3*x + 1)>"}︡
︠89c79a0b-b241-4dbc-8de5-91f3654fbd69︠
P.an_element().factor()
︡fd12aca7-9eb4-4b2b-aee4-03918e2001a0︡{"stdout": "(x + 1)^-3 * x^2"}︡
︠fbfc4a83-56c1-471d-9213-24a5ed348e94i︠
%html
<p>Last, we observe that the new test has automatically become part of the test suite. We remark that the existing test became more serious as well, thanks to an_element() returning something more interesting.</p>

︡46001f21-8971-46db-afc7-221e55674cf8︡{"html": "<p>Last, we observe that the new test has automatically become part of the test suite. We remark that the existing test became more serious as well, thanks to an_element() returning something more interesting.</p>"}︡
︠8e5f6a39-26e4-4dde-93ca-d0c0df61ef27︠
TestSuite(P).run(verbose=True)
︡d681143d-7449-4087-8c70-6f99ef9b0403︡{"stdout": "running ._test_additive_associativity() . . . pass\nrunning ._test_an_element() . . . pass\nrunning ._test_associativity() . . . pass\nrunning ._test_category() . . . pass\nrunning ._test_distributivity() . . . pass\nrunning ._test_elements() . . .\n  Running the test suite of self.an_element()\n  running ._test_category() . . . pass\n  running ._test_eq() . . . pass\n  running ._test_factorisation() . . . pass\n  running ._test_not_implemented_methods() . . . pass\n  running ._test_pickling() . . . pass\n  pass\nrunning ._test_elements_eq() . . . pass\nrunning ._test_eq() . . . pass\nrunning ._test_not_implemented_methods() . . . pass\nrunning ._test_one() . . . pass\nrunning ._test_pickling() . . . pass\nrunning ._test_prod() . . . pass\nrunning ._test_some_elements() . . . pass\nrunning ._test_zero() . . . pass"}︡
